# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for server preferences.
"""

import os
import sys
import re
import uuid
import shutil

from gns3.qt import QtNetwork, QtWidgets
from ..ui.server_preferences_page_ui import Ui_ServerPreferencesPageWidget
from ..servers import Servers
from ..topology import Topology
from ..utils.message_box import MessageBox
from ..utils.progress_dialog import ProgressDialog
from ..utils.wait_for_connection_worker import WaitForConnectionWorker
from ..utils.wait_for_vm_worker import WaitForVMWorker
from ..settings import SERVERS_SETTINGS
from ..gns3_vm import GNS3VM


class ServerPreferencesPage(QtWidgets.QWidget, Ui_ServerPreferencesPageWidget):

    """
    QWidget configuration page for server preferences.
    """

    def __init__(self, parent=None):

        super().__init__()
        self.setupUi(self)
        self._remote_servers = {}

        # connect the slots
        self.uiServerPreferenceTabWidget.currentChanged.connect(self._tabChangedSlot)
        self.uiLocalServerToolButton.clicked.connect(self._localServerBrowserSlot)
        self.uiUbridgeToolButton.clicked.connect(self._ubridgeBrowserSlot)
        self.uiAddRemoteServerPushButton.clicked.connect(self._remoteServerAddSlot)
        self.uiDeleteRemoteServerPushButton.clicked.connect(self._remoteServerDeleteSlot)
        self.uiRemoteServersTreeWidget.itemClicked.connect(self._remoteServerClickedSlot)
        self.uiRemoteServersTreeWidget.itemSelectionChanged.connect(self._remoteServerChangedSlot)
        self.uiRestoreDefaultsPushButton.clicked.connect(self._restoreDefaultsSlot)
        self.uiLocalServerAutoStartCheckBox.stateChanged.connect(self._useLocalServerAutoStartSlot)
        self.uiEnableVMCheckBox.stateChanged.connect(self._enableGNS3VMSlot)
        self.uiRefreshPushButton.clicked.connect(self._refreshVMListSlot)
        self.uiVmwareRadioButton.clicked.connect(self._listVMwareVMsSlot)
        self.uiVirtualBoxRadioButton.clicked.connect(self._listVirtualBoxVMsSlot)

        # load all available addresses
        for address in QtNetwork.QNetworkInterface.allAddresses():
            address_string = address.toString()
            # if address.protocol() == QtNetwork.QAbstractSocket.IPv6Protocol:
            # we do not want the scope id when using an IPv6 address...
            # address.setScopeId("")
            self.uiLocalServerHostComboBox.addItem(address_string, address.toString())

        # default is 127.0.0.1
        index = self.uiLocalServerHostComboBox.findText("127.0.0.1")
        if index != -1:
            self.uiLocalServerHostComboBox.setCurrentIndex(index)

    def _tabChangedSlot(self, index):
        if index == 1:
            self._refreshVMListSlot()

    def _listVMwareVMsSlot(self):
        """
        Slot to refresh the VMware VMs list.
        """

        self._refreshVMListSlot()

    def _listVirtualBoxVMsSlot(self):
        """
        Slot to refresh the VirtualBox VMs list.
        """

        self._refreshVMListSlot()

    def _refreshVMListSlot(self):
        """
        Refresh the list of VM available in VMware or VirtualBox.
        """

        if not self.uiEnableVMCheckBox.isChecked():
            return
        server = Servers.instance().localServer()
        if self.uiVmwareRadioButton.isChecked():
            server.get("/vmware/vms", self._getVMsFromServerCallback)
        elif self.uiVirtualBoxRadioButton.isChecked():
            server.get("/virtualbox/vms", self._getVMsFromServerCallback)

    def _getVMsFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getVMsFromServer.

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "VM List", "{}".format(result["message"]))
        else:
            self.uiVMListComboBox.clear()
            for vm in result:
                self.uiVMListComboBox.addItem(vm["vmname"], vm.get("vmx_path", ""))
            gns3_vm = Servers.instance().vmSettings()
            index = self.uiVMListComboBox.findText(gns3_vm["vmname"])
            if index != -1:
                self.uiVMListComboBox.setCurrentIndex(index)
            else:
                index = self.uiVMListComboBox.findText("GNS3 VM")
                if index != -1:
                    self.uiVMListComboBox.setCurrentIndex(index)

    def _enableGNS3VMSlot(self, state):
        """
        Slot to enable or not the GNS3 VM settings.
        """

        if state:
            if not self.uiLocalServerAutoStartCheckBox.isChecked():
                QtWidgets.QMessageBox.critical(self, "GNS3 VM", "The local server need to be enable in order to use the GNS3 VM")
                self.uiEnableVMCheckBox.setChecked(False)
                return
            self.uiGNS3VMSettingsGroupBox.setEnabled(True)
            self._refreshVMListSlot()
        else:
            self.uiGNS3VMSettingsGroupBox.setEnabled(False)


    def _useLocalServerAutoStartSlot(self, state):
        """
        Slot to enable or not local server settings.
        """

        if state:
            self.uiGeneralSettingsGroupBox.setEnabled(True)
            self.uiConsolePortRangeGroupBox.setEnabled(True)
            self.uiUDPPortRangeGroupBox.setEnabled(True)
        else:
            if self.uiEnableVMCheckBox.isChecked():
                QtWidgets.QMessageBox.critical(self, "GNS3 VM", "The local server need to be enable in order to use the GNS3 VM. Please turn off the GNS3 VM before turning off the local server.")
                self.uiLocalServerAutoStartCheckBox.setChecked(True)
                return
            self.uiGeneralSettingsGroupBox.setEnabled(False)
            self.uiConsolePortRangeGroupBox.setEnabled(False)
            self.uiUDPPortRangeGroupBox.setEnabled(False)

    def _restoreDefaultsSlot(self):
        """
        Slot to restore default settings
        """

        self._populateWidgets(SERVERS_SETTINGS)

    def _localServerBrowserSlot(self):
        """
        Slot to open a file browser and select a local server.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"
        server_path = shutil.which("gns3server")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select the local server", server_path, filter)
        if not path:
            return

        self.uiLocalServerPathLineEdit.setText(path)

    def _ubridgeBrowserSlot(self):
        """
        Slot to open a file browser and select the ubridge executable path.
        """

        filter = ""
        if sys.platform.startswith("win"):
            filter = "Executable (*.exe);;All files (*.*)"

        ubridge_path = shutil.which("ubridge")
        path, _ = QtWidgets.QFileDialog.getOpenFileName(self, "Select ubridge executable", ubridge_path, filter)
        if not path:
            return

        self.uiUbridgePathLineEdit.setText(path)

    def _remoteServerClickedSlot(self, item, column):
        """
        Loads a selected remote server from the tree widget.

        :param item: selected QTreeWidgetItem instance
        :param column: ignored
        """

        settings = item.settings
        protocol = settings["protocol"].upper()
        try:
            port = int(settings["port"])
        except ValueError:
            QtWidgets.QMessageBox.critical(self, "Remote server", "Invalid port")
            return
        self.uiRemoteServerProtocolComboBox.setCurrentIndex(self.uiRemoteServerProtocolComboBox.findText(protocol))
        self.uiRemoteServerPortLineEdit.setText(settings["host"])
        self.uiRemoteServerPortSpinBox.setValue(port)
        self.uiRAMLimitSpinBox.setValue(settings["ram_limit"])
        self.uiRemoteServerUserLineEdit.setText(settings["user"])

    def _remoteServerChangedSlot(self):
        """
        Enables the use of the delete button.
        """

        item = self.uiRemoteServersTreeWidget.currentItem()
        if item:
            self.uiDeleteRemoteServerPushButton.setEnabled(True)
        else:
            self.uiDeleteRemoteServerPushButton.setEnabled(False)

    def _remoteServerAddSlot(self):
        """
        Adds a new remote server.
        """

        protocol = self.uiRemoteServerProtocolComboBox.currentText().lower()
        host = self.uiRemoteServerPortLineEdit.text().strip()
        port = self.uiRemoteServerPortSpinBox.value()
        ram_limit = self.uiRAMLimitSpinBox.value()
        user = self.uiRemoteServerUserLineEdit.text().strip()
        password = self.uiRemoteServerPasswordLineEdit.text().strip()

        if not re.match(r"^[a-zA-Z0-9\.{}-]+$".format("\u0370-\u1CDF\u2C00-\u30FF\u4E00-\u9FBF"), host):
            QtWidgets.QMessageBox.critical(self, "Remote server", "Invalid remote server hostname {}".format(host))
            return
        if port is None or port < 1:
            QtWidgets.QMessageBox.critical(self, "Remote server", "Invalid remote server port {}".format(port))
            return

        # check if the remote server is already defined
        for server in self._remote_servers.values():
            if server["protocol"] == protocol and server["host"] == host and server["port"] == port and server["user"] == user:
                QtWidgets.QMessageBox.critical(self, "Remote server", "Remote server is already defined.")
                return

        settings = {"protocol": protocol,
                    "host": host,
                    "port": port,
                    "ram_limit": ram_limit,
                    "user": user,
                    "password": password}

        # add a new entry in the tree widget
        item = QtWidgets.QTreeWidgetItem(self.uiRemoteServersTreeWidget)
        item.setText(0, protocol)
        item.setText(1, host)
        item.setText(2, str(port))
        item.setText(3, user)
        item.settings = settings
        item.server_id = uuid.uuid4()  # Create a temporary unique server id

        # keep track of this remote server
        self._remote_servers[item.server_id] = settings

        self.uiRemoteServerPortSpinBox.setValue(self.uiRemoteServerPortSpinBox.value() + 1)
        self.uiRemoteServersTreeWidget.resizeColumnToContents(0)

    def _remoteServerDeleteSlot(self):
        """
        Deletes a remote server.
        """

        item = self.uiRemoteServersTreeWidget.currentItem()
        if item:
            assert item.server_id in self._remote_servers, "Missing {} in {}".format(item.server_id, self._remote_servers)
            del self._remote_servers[item.server_id]
            self.uiRemoteServersTreeWidget.takeTopLevelItem(self.uiRemoteServersTreeWidget.indexOfTopLevelItem(item))

    def _populateWidgets(self, servers_settings):
        """
        Populates the widgets with the settings.

        :param servers_settings: servers settings
        """

        if servers_settings["load_balancing_method"] == "ram_usage":
            self.uiRAMUsageRadioButton.setChecked(True)
        elif servers_settings["load_balancing_method"] == "round_robin":
            self.uiRoundRobinRadioButton.setChecked(True)
        elif servers_settings["load_balancing_method"] == "rendezvous_hashing":
            self.uiRendezVousHashingRadioButton.setChecked(True)

        # local server settings
        local_server_settings = servers_settings["local_server"]
        self.uiLocalServerPathLineEdit.setText(local_server_settings["path"])
        self.uiUbridgePathLineEdit.setText(local_server_settings["ubridge_path"])
        index = self.uiLocalServerHostComboBox.findData(local_server_settings["host"])
        if index != -1:
            self.uiLocalServerHostComboBox.setCurrentIndex(index)
        self.uiLocalServerPortSpinBox.setValue(local_server_settings["port"])
        self.uiLocalServerAutoStartCheckBox.setChecked(local_server_settings["auto_start"])
        self.uiLocalServerAuthCheckBox.setChecked(local_server_settings["auth"])
        self.uiConsoleConnectionsToAnyIPCheckBox.setChecked(local_server_settings["allow_console_from_anywhere"])
        self.uiConsoleStartPortSpinBox.setValue(local_server_settings["console_start_port_range"])
        self.uiConsoleEndPortSpinBox.setValue(local_server_settings["console_end_port_range"])
        self.uiUDPStartPortSpinBox.setValue(local_server_settings["udp_start_port_range"])
        self.uiUDPEndPortSpinBox.setValue(local_server_settings["udp_end_port_range"])

        # GNS3 VM settings
        vm_settings = servers_settings["vm"]
        self.uiVMUserLineEdit.setText(vm_settings["user"])
        self.uiVMPasswordLineEdit.setText(vm_settings["password"])
        self.uiEnableVMCheckBox.setChecked(vm_settings["auto_start"])
        self.uiShutdownCheckBox.setChecked(vm_settings["auto_stop"])
        self.uiAdjustLocalServerIPCheckBox.setChecked(vm_settings["adjust_local_server_ip"])
        index = self.uiVMListComboBox.findText(vm_settings["vmname"])
        if index != -1:
            self.uiVMListComboBox.setCurrentIndex(index)
        else:
            self.uiVMListComboBox.clear()
            self.uiVMListComboBox.addItem(vm_settings["vmname"], vm_settings["vmx_path"])
        if vm_settings["virtualization"] == "VMware":
            self.uiVmwareRadioButton.setChecked(True)
        elif vm_settings["virtualization"] == "VirtualBox":
            self.uiVirtualBoxRadioButton.setChecked(True)
        self.uiHeadlessCheckBox.setChecked(vm_settings["headless"])

    def loadPreferences(self):
        """
        Loads the server preferences.
        """

        servers = Servers.instance()
        # load the servers settings
        servers_settings = servers.settings()
        self._populateWidgets(servers_settings)

        # load remote server preferences
        self._remote_servers.clear()
        self.uiRemoteServersTreeWidget.clear()
        for server_id, server in servers.remoteServers().items():
            protocol = server.protocol()
            host = server.host()
            port = server.port()
            user = server.user()
            self._remote_servers[server_id] = server.settings()
            item = QtWidgets.QTreeWidgetItem(self.uiRemoteServersTreeWidget)
            item.setText(0, protocol)
            item.setText(1, host)
            item.setText(2, str(port))
            item.setText(3, user)
            item.settings = server.settings()
            item.server_id = server_id

        self.uiRemoteServersTreeWidget.resizeColumnToContents(0)

    def savePreferences(self):
        """
        Saves the server preferences.
        """

        servers = Servers.instance()
        servers_settings = servers.settings()
        local_server_settings = servers_settings["local_server"]
        restart_local_server = False
        restart_gns3_vm = False

        # save the local server preferences
        new_local_server_settings = local_server_settings.copy()
        new_local_server_settings.update({"path": self.uiLocalServerPathLineEdit.text(),
                                          "ubridge_path": self.uiUbridgePathLineEdit.text(),
                                          "host": self.uiLocalServerHostComboBox.itemData(self.uiLocalServerHostComboBox.currentIndex()),
                                          "port": self.uiLocalServerPortSpinBox.value(),
                                          "auto_start": self.uiLocalServerAutoStartCheckBox.isChecked(),
                                          "allow_console_from_anywhere": self.uiConsoleConnectionsToAnyIPCheckBox.isChecked(),
                                          "auth": self.uiLocalServerAuthCheckBox.isChecked(),
                                          "console_start_port_range": self.uiConsoleStartPortSpinBox.value(),
                                          "console_end_port_range": self.uiConsoleEndPortSpinBox.value(),
                                          "udp_start_port_range": self.uiUDPStartPortSpinBox.value(),
                                          "udp_end_port_range": self.uiUDPEndPortSpinBox.value()})

        if new_local_server_settings["console_end_port_range"] <= new_local_server_settings["console_start_port_range"]:
            QtWidgets.QMessageBox.critical(self, "Port range", "Invalid console port range from {} to {}".format(new_local_server_settings["console_start_port_range"],
                                                                                                                 new_local_server_settings["console_end_port_range"]))
            return

        if new_local_server_settings["udp_end_port_range"] <= new_local_server_settings["udp_start_port_range"]:
            QtWidgets.QMessageBox.critical(self, "Port range", "Invalid UDP port range from {} to {}".format(new_local_server_settings["udp_start_port_range"],
                                                                                                             new_local_server_settings["udp_end_port_range"]))
            return

        if new_local_server_settings["auto_start"]:
            if not os.path.isfile(new_local_server_settings["path"]):
                QtWidgets.QMessageBox.critical(self, "Local server", "Could not find local server {}".format(new_local_server_settings["path"]))
                return
            if not os.access(new_local_server_settings["path"], os.X_OK):
                QtWidgets.QMessageBox.critical(self, "Local server", "{} is not an executable".format(new_local_server_settings["path"]))

            if new_local_server_settings != local_server_settings:
                # first check if we have nodes on the local server
                local_nodes = []
                topology = Topology.instance()
                for node in topology.nodes():
                    if node.server().isLocal():
                        local_nodes.append(node.name())
                if local_nodes:
                    nodes = "\n".join(local_nodes)
                    MessageBox(self, "Local server", "Please close your project or delete all the nodes running on the \
                    local server before changing the local server settings", nodes)
                    return
                servers.setLocalServerSettings(new_local_server_settings)
                servers.registerLocalServer()
                restart_local_server = True
        else:
            servers.setLocalServerSettings(new_local_server_settings)
            servers.stopLocalServer(wait=True)

        # save the GNS3 VM preferences
        new_gns3vm_settings = servers_settings["vm"].copy()
        new_gns3vm_settings.update({"auto_start": self.uiEnableVMCheckBox.isChecked(),
                                    "auto_stop": self.uiShutdownCheckBox.isChecked(),
                                    "adjust_local_server_ip": self.uiAdjustLocalServerIPCheckBox.isChecked(),
                                    "vmname": self.uiVMListComboBox.currentText(),
                                    "vmx_path": self.uiVMListComboBox.currentData(),
                                    "headless": self.uiHeadlessCheckBox.isChecked(),
                                    "user": self.uiVMUserLineEdit.text(),
                                    "password": self.uiVMPasswordLineEdit.text()})
        if self.uiVmwareRadioButton.isChecked():
            new_gns3vm_settings["virtualization"] = "VMware"
        elif self.uiVirtualBoxRadioButton.isChecked():
            new_gns3vm_settings["virtualization"] = "VirtualBox"
        if new_gns3vm_settings != servers_settings["vm"]:
            restart_gns3_vm = True
        servers_settings["vm"].update(new_gns3vm_settings)

        # save the load-balancing preference
        if self.uiRAMUsageRadioButton.isChecked():
            servers_settings["load_balancing_method"] = "ram_usage"
        elif self.uiRoundRobinRadioButton.isChecked():
            servers_settings["load_balancing_method"] = "round_robin"
        elif self.uiRendezVousHashingRadioButton.isChecked():
            servers_settings["load_balancing_method"] = "rendezvous_hashing"

        # save the server preferences
        servers.setSettings(servers_settings)
        # save the remote server preferences
        servers.updateRemoteServers(self._remote_servers)
        servers.save()

        # start or restart the GNS3 VM if required
        if restart_gns3_vm:
            gns3_vm = GNS3VM.instance()
            gns3_vm.shutdown(force=True)
            if gns3_vm.autoStart():
                servers.initVMServer()
                worker = WaitForVMWorker()
                progress_dialog = ProgressDialog(worker, "GNS3 VM", "Starting the GNS3 VM...", "Cancel", busy=True, parent=self)
                progress_dialog.show()
                if progress_dialog.exec_():
                    local_server_ip = gns3_vm.adjustLocalServerIP()
                    if local_server_ip != new_local_server_settings["host"]:
                        new_local_server_settings["host"] = local_server_ip
                        index = self.uiLocalServerHostComboBox.findData(local_server_ip)
                        if index != -1:
                            restart_local_server = True
                            self.uiLocalServerHostComboBox.setCurrentIndex(index)

        # start or restart the local server if required
        if restart_local_server:
            servers.stopLocalServer(wait=True)
            if servers.startLocalServer():
                worker = WaitForConnectionWorker(new_local_server_settings["host"], new_local_server_settings["port"])
                dialog = ProgressDialog(worker, "Local server", "Connecting...", "Cancel", busy=True, parent=self)
                dialog.show()
                dialog.exec_()
            else:
                QtWidgets.QMessageBox.critical(self, "Local server", "Could not start the local server process: {}".format(new_local_server_settings["path"]))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/vpcs/ui/vpcs_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VPCSPreferencesPageWidget(object):

    def setupUi(self, VPCSPreferencesPageWidget):
        VPCSPreferencesPageWidget.setObjectName("VPCSPreferencesPageWidget")
        VPCSPreferencesPageWidget.resize(469, 392)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(VPCSPreferencesPageWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiTabWidget = QtWidgets.QTabWidget(VPCSPreferencesPageWidget)
        self.uiTabWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralSettingsTabWidget = QtWidgets.QWidget()
        self.uiGeneralSettingsTabWidget.setObjectName("uiGeneralSettingsTabWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiGeneralSettingsTabWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiUseLocalServercheckBox = QtWidgets.QCheckBox(self.uiGeneralSettingsTabWidget)
        self.uiUseLocalServercheckBox.setChecked(True)
        self.uiUseLocalServercheckBox.setObjectName("uiUseLocalServercheckBox")
        self.verticalLayout.addWidget(self.uiUseLocalServercheckBox)
        self.uiVPCSPathLabel = QtWidgets.QLabel(self.uiGeneralSettingsTabWidget)
        self.uiVPCSPathLabel.setObjectName("uiVPCSPathLabel")
        self.verticalLayout.addWidget(self.uiVPCSPathLabel)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiVPCSPathLineEdit = QtWidgets.QLineEdit(self.uiGeneralSettingsTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVPCSPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVPCSPathLineEdit.setSizePolicy(sizePolicy)
        self.uiVPCSPathLineEdit.setObjectName("uiVPCSPathLineEdit")
        self.horizontalLayout_5.addWidget(self.uiVPCSPathLineEdit)
        self.uiVPCSPathToolButton = QtWidgets.QToolButton(self.uiGeneralSettingsTabWidget)
        self.uiVPCSPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiVPCSPathToolButton.setObjectName("uiVPCSPathToolButton")
        self.horizontalLayout_5.addWidget(self.uiVPCSPathToolButton)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        spacerItem = QtWidgets.QSpacerItem(390, 193, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.uiTabWidget.addTab(self.uiGeneralSettingsTabWidget, "")
        self.uiVPCSTabWidget = QtWidgets.QWidget()
        self.uiVPCSTabWidget.setObjectName("uiVPCSTabWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.uiVPCSTabWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiScriptFileEdit = QtWidgets.QLineEdit(self.uiVPCSTabWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiScriptFileEdit.sizePolicy().hasHeightForWidth())
        self.uiScriptFileEdit.setSizePolicy(sizePolicy)
        self.uiScriptFileEdit.setObjectName("uiScriptFileEdit")
        self.horizontalLayout_6.addWidget(self.uiScriptFileEdit)
        self.uiScriptFileToolButton = QtWidgets.QToolButton(self.uiVPCSTabWidget)
        self.uiScriptFileToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiScriptFileToolButton.setObjectName("uiScriptFileToolButton")
        self.horizontalLayout_6.addWidget(self.uiScriptFileToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_6, 0, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiVPCSTabWidget)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout.addWidget(self.uiSymbolLabel, 3, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiVPCSTabWidget)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiVPCSTabWidget)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout.addLayout(self.horizontalLayout_7, 3, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiVPCSTabWidget)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout.addWidget(self.uiCategoryLabel, 4, 0, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiVPCSTabWidget)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout.addWidget(self.uiCategoryComboBox, 4, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 387, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem1, 5, 0, 1, 2)
        self.uiScriptFileLabel = QtWidgets.QLabel(self.uiVPCSTabWidget)
        self.uiScriptFileLabel.setObjectName("uiScriptFileLabel")
        self.gridLayout.addWidget(self.uiScriptFileLabel, 0, 0, 1, 1)
        self.uiTabWidget.addTab(self.uiVPCSTabWidget, "")
        self.verticalLayout_2.addWidget(self.uiTabWidget)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem2 = QtWidgets.QSpacerItem(138, 17, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(VPCSPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_2.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.retranslateUi(VPCSPreferencesPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(VPCSPreferencesPageWidget)

    def retranslateUi(self, VPCSPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VPCSPreferencesPageWidget.setWindowTitle(_translate("VPCSPreferencesPageWidget", "VPCS"))
        self.uiUseLocalServercheckBox.setText(_translate("VPCSPreferencesPageWidget", "Use the local server"))
        self.uiVPCSPathLabel.setText(_translate("VPCSPreferencesPageWidget", "Path to VPCS executable:"))
        self.uiVPCSPathToolButton.setText(_translate("VPCSPreferencesPageWidget", "&Browse..."))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralSettingsTabWidget), _translate("VPCSPreferencesPageWidget", "General settings"))
        self.uiScriptFileToolButton.setText(_translate("VPCSPreferencesPageWidget", "&Browse..."))
        self.uiSymbolLabel.setText(_translate("VPCSPreferencesPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("VPCSPreferencesPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("VPCSPreferencesPageWidget", "Category:"))
        self.uiScriptFileLabel.setText(_translate("VPCSPreferencesPageWidget", "Base script file:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiVPCSTabWidget), _translate("VPCSPreferencesPageWidget", "VPCS VM settings"))
        self.uiRestoreDefaultsPushButton.setText(_translate("VPCSPreferencesPageWidget", "Restore defaults"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from unittest.mock import patch, Mock


def test_allocateUDPPort(vpcs_device):

    with patch('gns3.node.Node.httpPost') as mock:
        vpcs_device.allocateUDPPort(1)
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/ports/udp"

        # Connect the signal
        signal_mock = Mock()
        vpcs_device.allocate_udp_nio_signal.connect(signal_mock)

        # Callback
        args[1]({"udp_port": 4242}, context=kwargs["context"])

        # Check the signal
        assert signal_mock.called
        args, kwargs = signal_mock.call_args
        assert args[0] == vpcs_device.id()
        assert args[1] == 1
        assert args[2] == 4242



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import pytest
import os
from uuid import uuid4
from unittest.mock import patch, Mock

from gns3.modules.iou.iou_device import IOUDevice
from gns3.ports.port import Port
from gns3.nios.nio_udp import NIOUDP
from gns3.node import Node
from gns3.utils.normalize_filename import normalize_filename
from gns3.modules.iou import IOU


@pytest.fixture
def fake_bin(tmpdir):
    path = str(tmpdir / "test.bin")
    with open(path, "w+") as f:
        f.write("1")
    return path


@pytest.fixture
def fake_iourc(tmpdir):
    path = str(tmpdir / "iourc")
    with open(path, "wb+") as f:
        f.write(b"[license]\r\ngns42 = dsfdsfdsfdsf;\r\n")
    return path


def test_iou_device_init(local_server, project):

    iou_device = IOUDevice(None, local_server, project)


def test_iou_device_setup(iou_device, project, fake_iourc):

    with patch('gns3.node.Node.httpPost') as mock:
        iou_device._module._settings["iourc_path"] = fake_iourc

        iou_device.setup("/tmp/iou.bin", name="PC 1")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms"
        assert kwargs["body"] == {
            "name": "PC 1",
            "path": "/tmp/iou.bin",
            "iourc_content": "[license]\r\ngns42 = dsfdsfdsfdsf;\r\n"
        }

        # Callback
        params = {
            "console": 2000,
            "name": "PC1",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "path": "iou.bin",
            "md5sum": "0cc175b9c0f1b6a831c399e269772661"
        }
        args[1](params)

        assert iou_device.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"


def test_iou_device_setup_md5_missing(iou_device, project, fake_iourc):
    """
    It should notify the user asking him to upload the image
    """

    with patch('gns3.node.Node.httpPost') as mock:
        iou_device._module._settings["iourc_path"] = fake_iourc

        iou_device.setup("/tmp/iou.bin", name="PC 1")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms"
        assert kwargs["body"] == {
            "name": "PC 1",
            "path": "/tmp/iou.bin",
            "iourc_content": "[license]\r\ngns42 = dsfdsfdsfdsf;\r\n"
        }

        # Callback
        params = {
            "console": 2000,
            "name": "PC1",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "path": "iou.bin",
        }

        with patch("gns3.image_manager.ImageManager.addMissingImage") as mock:
            args[1](params)
            assert mock.called

        assert iou_device.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"


def test_iou_device_setup_with_uuid(iou_device, project, fake_iourc):
    """
    If we have an ID that mean the VM already exits and we should not send startup_script
    """

    with patch('gns3.node.Node.httpPost') as mock:
        iou_device._module._settings["iourc_path"] = fake_iourc

        iou_device.setup("/tmp/iou.bin", name="PC 1", vm_id="aec7a00c-e71c-45a6-8c04-29e40732883c")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms"
        assert kwargs["body"] == {
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "name": "PC 1",
            "path": "/tmp/iou.bin",
            "iourc_content": "[license]\r\ngns42 = dsfdsfdsfdsf;\r\n"
        }

        # Callback
        params = {
            "console": 2000,
            "name": "PC1",
            "project_id": "f91bd115-3b5c-402e-b411-e5919723cf4b",
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "path": "iou.bin",
            "md5sum": "0cc175b9c0f1b6a831c399e269772661"
        }
        args[1](params)

        assert iou_device.vm_id() == "aec7a00c-e71c-45a6-8c04-29e40732883c"


def test_iou_device_setup_with_startup_config(iou_device, project, tmpdir, fake_iourc):
    """
    If we have an ID that mean the VM already exits and we should not send startup_script
    """

    startup_config = str(tmpdir / "config.cfg")
    with open(startup_config, "w+") as f:
        f.write("hostname %h")

    with patch('gns3.node.Node.httpPost') as mock:
        iou_device._module._settings["iourc_path"] = fake_iourc

        iou_device.setup("/tmp/iou.bin", name="PC 1", vm_id="aec7a00c-e71c-45a6-8c04-29e40732883c", additional_settings={"startup_config": startup_config})
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms"
        assert kwargs["body"] == {
            "vm_id": "aec7a00c-e71c-45a6-8c04-29e40732883c",
            "name": "PC 1",
            "path": "/tmp/iou.bin",
            "startup_config_content": "hostname %h",
            'iourc_content': '[license]\r\ngns42 = dsfdsfdsfdsf;\r\n'
        }


def test_update(iou_device):

    new_settings = {
        "name": "Unreal IOU",
    }

    with patch('gns3.node.Node.httpPut') as mock:
        iou_device.update(new_settings)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}".format(vm_id=iou_device.vm_id())
        assert kwargs["body"] == new_settings

        # Callback
        args[1]({})


def test_update_startup_config(iou_device, tmpdir):

    startup_config = str(tmpdir / "config.cfg")
    with open(startup_config, "w+") as f:
        f.write("hostname %h")

    new_settings = {
        "name": "Unreal IOU",
        "startup_config": startup_config
    }

    with patch('gns3.node.Node.httpPut') as mock:
        iou_device.update(new_settings)

        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}".format(vm_id=iou_device.vm_id())
        assert kwargs["body"]["name"] == "Unreal IOU"
        assert kwargs["body"]["startup_config_content"] == "hostname %h"

        # Callback
        args[1]({})


def test_dump(local_server, project):

    iou_device = IOUDevice(IOU(), local_server, project)
    iou_device._settings["name"] = "IOU 1"
    iou_device._settings["path"] = "test.bin"
    iou_device._settings["startup_config"] = "/tmp"
    iou_device._settings["ethernet_adapters"] = 2
    iou_device._settings["serial_adapters"] = 2
    iou_device._settings["l1_keepalives"] = True
    iou_device._settings["ram"] = 256
    iou_device._settings["nvram"] = 128
    iou_device._settings["use_default_iou_values"] = True
    iou_device._settings["md5sum"] = "0cc175b9c0f1b6a831c399e269772661"
    iou_device._addAdapters(iou_device._settings["ethernet_adapters"], iou_device._settings["serial_adapters"])

    assert iou_device.dump() == {
        "description": "IOU device",
        "id": iou_device.id(),
        "ports": [
            {"adapter_number": 0,
             "id": 1,
             "name": "Ethernet0/0",
             "port_number": 0},
            {"adapter_number": 0,
             "id": 2,
             "name": "Ethernet0/1",
             "port_number": 1},
            {"adapter_number": 0,
             "id": 3,
             "name": "Ethernet0/2",
             "port_number": 2},
            {"adapter_number": 0,
             "id": 4,
             "name": "Ethernet0/3",
             "port_number": 3},
            {"adapter_number": 1,
             "id": 5,
             "name": "Ethernet1/0",
             "port_number": 0},
            {"adapter_number": 1,
             "id": 6,
             "name": "Ethernet1/1",
             "port_number": 1},
            {"adapter_number": 1,
             "id": 7,
             "name": "Ethernet1/2",
             "port_number": 2},
            {"adapter_number": 1,
             "id": 8,
             "name": "Ethernet1/3",
             "port_number": 3},
            {"adapter_number": 2,
             "id": 9,
             "name": "Serial2/0",
             "port_number": 0},
            {"adapter_number": 2,
             "id": 10,
             "name": "Serial2/1",
             "port_number": 1},
            {"adapter_number": 2,
             "id": 11,
             "name": "Serial2/2",
             "port_number": 2},
            {"adapter_number": 2,
             "id": 12,
             "name": "Serial2/3",
             "port_number": 3},
            {"adapter_number": 3,
             "id": 13,
             "name": "Serial3/0",
             "port_number": 0},
            {"adapter_number": 3,
             "id": 14,
             "name": "Serial3/1",
             "port_number": 1},
            {"adapter_number": 3,
             "id": 15,
             "name": "Serial3/2",
             "port_number": 2},
            {"adapter_number": 3,
             "id": 16,
             "name": "Serial3/3",
             "port_number": 3}
        ],
        "properties": {
            "name": "IOU 1",
            "path": "test.bin",
            "md5sum": "0cc175b9c0f1b6a831c399e269772661",
            "startup_config": "/tmp",
            "ethernet_adapters": 2,
            "serial_adapters": 2,
            "l1_keepalives": True,
            "ram": 256,
            "nvram": 128,
            "use_default_iou_values": True
        },
        "server_id": local_server.id(),
        "type": "IOUDevice",
        "vm_id": None
    }


def test_load(local_server, project, fake_bin):

    uuid = uuid4()
    iou_device = IOUDevice(IOU(), local_server, project)
    nio_node = {
        "description": "IOU device",
        "id": 1,
        "ports": [
            {
                "adapter_number": 0,
                "id": 1,
                "name": "Hyper Ethernet0/0",
                "port_number": 0},
        ],
        "properties": {
            "name": "IOU 1",
            "path": fake_bin,
            "startup_config": "/tmp",
            "ethernet_adapters": 1,
            "serial_adapters": 0,
        },
        "server_id": 1,
        "type": "IOUDevice",
        "vm_id": uuid
    }
    with patch("gns3.modules.iou.iou_device.IOUDevice.setup") as mock:

        iou_device.load(nio_node)
        iou_device._addAdapters(1, 0)

        assert mock.called
        (path, name, vm_id, settings), kwargs = mock.call_args
        assert path == fake_bin
        assert name == "IOU 1"
        assert settings == {"ethernet_adapters": 1, "serial_adapters": 0, "startup_config": "/tmp"}
        assert vm_id == uuid

    iou_device.loaded_signal.emit()
    assert iou_device._ports[0].name() == "Hyper Ethernet0/0"


def test_load_1_2(local_server, project, fake_bin):

    uuid = uuid4()
    iou_device = IOUDevice(IOU(), local_server, project)
    nio_node = {
        "description": "IOU device",
        "id": 1,
        "ports": [
            {
                "slot_number": 0,
                "id": 1,
                "name": "Hyper Ethernet0/0",
                "port_number": 0},
        ],
        "properties": {
            "name": "IOU 1",
            "path": fake_bin,
            "startup_config": "/tmp",
            "ethernet_adapters": 1,
            "serial_adapters": 0,
        },
        "server_id": 1,
        "type": "IOUDevice",
        "vm_id": uuid
    }
    with patch("gns3.modules.iou.iou_device.IOUDevice.setup") as mock:
        iou_device.load(nio_node)
        iou_device._addAdapters(1, 0)

        assert mock.called
        (path, name, vm_id, settings), kwargs = mock.call_args
        assert path == fake_bin
        assert name == "IOU 1"
        assert settings == {"ethernet_adapters": 1, "serial_adapters": 0, "startup_config": "/tmp"}
        assert vm_id == uuid

    iou_device.loaded_signal.emit()
    assert iou_device._ports[0].name() == "Hyper Ethernet0/0"


def test_startPacketCapture(iou_device):

    port = Port("test")
    port.setAdapterNumber(2)
    port.setPortNumber(1)

    with patch("gns3.node.Node.httpPost") as mock:
        iou_device.startPacketCapture(port, "test.pcap", "DLT_EN10MB")
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}/adapters/2/ports/1/start_capture".format(vm_id=iou_device.vm_id())
        assert kwargs["body"] == {
            "data_link_type": "DLT_EN10MB",
            "capture_file_name": "test.pcap"
        }

        with patch("gns3.ports.port.Port.startPacketCapture") as port_mock:

            # Callback
            args[1]({"pcap_file_path": "/tmp/test.pcap"}, context=kwargs["context"])

            assert port_mock.called_with("/tmp/test.pcap")


def test_stopPacketCapture(iou_device):

    port = Port("test")
    port.setAdapterNumber(2)
    port.setPortNumber(1)

    with patch("gns3.node.Node.httpPost") as mock:
        iou_device.stopPacketCapture(port)
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}/adapters/2/ports/1/stop_capture".format(vm_id=iou_device.vm_id())

        with patch("gns3.ports.port.Port.stopPacketCapture") as port_mock:

            # Callback
            args[1]({}, context=kwargs["context"])

            assert port_mock.called


def test_exportConfig(iou_device, tmpdir):

    startup_path = str(tmpdir / "startup_config.cfg")
    private_path = str(tmpdir / "private_config.cfg")

    with patch("gns3.node.Node.httpGet") as mock:
        iou_device.exportConfig(startup_path, private_path)
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}/configs".format(vm_id=iou_device.vm_id())

        # Callback
        args[1]({"startup_config_content": "TEST", "private_config_content": "PRIVATE"}, context=kwargs["context"])

        with open(startup_path) as f:
            assert f.read() == "TEST"

        with open(private_path) as f:
            assert f.read() == "PRIVATE"


def test_exportConfigToDirectory(iou_device, tmpdir):

    path = str(tmpdir)

    with patch("gns3.node.Node.httpGet") as mock:
        iou_device.exportConfigToDirectory(path)
        assert mock.called
        args, kwargs = mock.call_args
        assert args[0] == "/iou/vms/{vm_id}/configs".format(vm_id=iou_device.vm_id())

        # Callback
        args[1]({"startup_config_content": "TEST"}, context=kwargs["context"])

        with open(os.path.join(path, normalize_filename(iou_device.name()) + "_startup-config.cfg")) as f:
            assert f.read() == "TEST"



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Configuration page for IOS router preferences.
"""

import os
import copy
import sys
import math
import zipfile
import logging

from gns3.qt import QtCore, QtGui, QtWidgets
from gns3.main_window import MainWindow
from gns3.dialogs.configuration_dialog import ConfigurationDialog
from gns3.utils.progress_dialog import ProgressDialog
from gns3.image_manager import ImageManager

from .. import Dynamips
from ..settings import IOS_ROUTER_SETTINGS
from ..utils.decompress_ios import isIOSCompressed
from ..utils.decompress_ios_worker import DecompressIOSWorker
from ..ui.ios_router_preferences_page_ui import Ui_IOSRouterPreferencesPageWidget
from ..pages.ios_router_configuration_page import IOSRouterConfigurationPage
from ..dialogs.ios_router_wizard import IOSRouterWizard


log = logging.getLogger(__name__)


class IOSRouterPreferencesPage(QtWidgets.QWidget, Ui_IOSRouterPreferencesPageWidget):

    """
    QWidget preference page for IOS routers.
    """

    _default_images_dir = ""

    def __init__(self):
        super().__init__()
        self.setupUi(self)

        self._main_window = MainWindow.instance()
        self._ios_routers = {}
        self._items = []

        self.uiNewIOSRouterPushButton.clicked.connect(self._iosRouterNewSlot)
        self.uiEditIOSRouterPushButton.clicked.connect(self._iosRouterEditSlot)
        self.uiDeleteIOSRouterPushButton.clicked.connect(self._iosRouterDeleteSlot)
        self.uiIOSRoutersTreeWidget.itemSelectionChanged.connect(self._iosRouterChangedSlot)
        self.uiDecompressIOSPushButton.clicked.connect(self._decompressIOSSlot)

    def _iosRouterChangedSlot(self):
        """
        Loads a selected an IOS router from the tree widget.
        """

        selection = self.uiIOSRoutersTreeWidget.selectedItems()
        self.uiDeleteIOSRouterPushButton.setEnabled(len(selection) != 0)
        single_selected = len(selection) == 1
        self.uiEditIOSRouterPushButton.setEnabled(single_selected)
        self.uiDecompressIOSPushButton.setEnabled(single_selected)

        if single_selected:
            key = selection[0].data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            self._refreshInfo(ios_router)
        else:
            self.uiIOSRouterInfoTreeWidget.clear()

    def _iosRouterNewSlot(self):
        """
        Creates a new IOS router.
        """

        wizard = IOSRouterWizard(self._ios_routers, parent=self)
        wizard.show()
        if wizard.exec_():

            ios_settings = wizard.getSettings()
            key = "{server}:{name}".format(server=ios_settings["server"], name=ios_settings["name"])

            self._ios_routers[key] = IOS_ROUTER_SETTINGS.copy()
            self._ios_routers[key].update(ios_settings)

            if ios_settings["platform"] == "c7200":
                self._ios_routers[key]["midplane"] = "vxr"
                self._ios_routers[key]["npe"] = "npe-400"
            else:
                self._ios_routers[key]["iomem"] = 5

            for slot_id in range(0, 7):
                slot = "slot{}".format(slot_id)
                if slot in ios_settings:
                    self._ios_routers[key][slot] = ios_settings[slot]

            for wic_id in range(0, 3):
                wic = "wic{}".format(wic_id)
                if wic in ios_settings:
                    self._ios_routers[key][wic] = ios_settings[wic]

            self._ios_routers[key].update(ios_settings)
            item = QtWidgets.QTreeWidgetItem(self.uiIOSRoutersTreeWidget)
            item.setText(0, self._ios_routers[key]["name"])
            item.setIcon(0, QtGui.QIcon(self._ios_routers[key]["symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)
            self.uiIOSRoutersTreeWidget.setCurrentItem(item)

    def _iosRouterEditSlot(self):
        """
        Edits an IOS router.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            dialog = ConfigurationDialog(ios_router["name"], ios_router, IOSRouterConfigurationPage(), parent=self)
            dialog.show()
            if dialog.exec_():
                # update the icon
                item.setIcon(0, QtGui.QIcon(ios_router["symbol"]))
                if ios_router["name"] != item.text(0):
                    # rename the IOS router
                    new_key = "{server}:{name}".format(server=ios_router["server"], name=ios_router["name"])
                    if new_key in self._ios_routers:
                        QtWidgets.QMessageBox.critical(self, "IOS router", "IOS router name {} already exists for server {}".format(ios_router["name"],
                                                                                                                                    ios_router["server"]))
                        ios_router["name"] = item.text(0)
                        return
                    self._ios_routers[new_key] = self._ios_routers[key]
                    del self._ios_routers[key]
                    item.setText(0, ios_router["name"])
                    item.setData(0, QtCore.Qt.UserRole, new_key)

                self._refreshInfo(ios_router)

    def _iosRouterDeleteSlot(self):
        """
        Deletes an IOS router.
        """

        for item in self.uiIOSRoutersTreeWidget.selectedItems():
            if item:
                key = item.data(0, QtCore.Qt.UserRole)

                del self._ios_routers[key]
                self.uiIOSRoutersTreeWidget.takeTopLevelItem(self.uiIOSRoutersTreeWidget.indexOfTopLevelItem(item))
                if self._ios_routers == {}:
                    self.uiEditIOSRouterPushButton.setEnabled(False)
                    self.uiDeleteIOSRouterPushButton.setEnabled(False)
                    self.uiDecompressIOSPushButton.setEnabled(False)

    def _imageUploadComplete(self):
        if self._upload_image_progress_dialog.wasCanceled():
            return
        self._upload_image_progress_dialog.accept()

    @staticmethod
    def getImageDirectory():
        return ImageManager.instance().getDirectoryForType("DYNAMIPS")

    @classmethod
    def getIOSImage(cls, parent, server):
        """

        :param parent: parent widget
        :param server: The server where the image is located

        :return: path to the IOS image or None
        """

        if not cls._default_images_dir:
            cls._default_images_dir = cls.getImageDirectory()

        path, _ = QtWidgets.QFileDialog.getOpenFileName(parent,
                                                        "Select an IOS image",
                                                        cls._default_images_dir,
                                                        "All files (*.*);;IOS image (*.bin *.image)",
                                                        "IOS image (*.bin *.image)")

        if not path:
            return
        cls._default_images_dir = os.path.dirname(path)

        if not os.access(path, os.R_OK):
            QtWidgets.QMessageBox.critical(parent, "IOS image", "Cannot read {}".format(path))
            return

        if sys.platform.startswith('win'):
            # Dynamips (Cygwin actually) doesn't like non ascii paths on Windows
            try:
                path.encode('ascii')
            except UnicodeEncodeError:
                QtWidgets.QMessageBox.warning(parent, "IOS image", "The IOS image filename should contains only ascii (English) characters.")

        try:
            with open(path, "rb") as f:
                # read the first 7 bytes of the file.
                elf_header_start = f.read(7)
        except OSError as e:
            QtWidgets.QMessageBox.critical(parent, "IOS image", "Cannot read ELF magic number: {}".format(e))
            return

        # file must start with the ELF magic number, be 32-bit, big endian and have an ELF version of 1
        if elf_header_start != b'\x7fELF\x01\x02\x01':
            QtWidgets.QMessageBox.critical(parent, "IOS image", "Sorry, this is not a valid IOS image!")
            return

        try:
            os.makedirs(cls.getImageDirectory(), exist_ok=True)
        except OSError as e:
            QtWidgets.QMessageBox.critical(parent, "IOS images directory", "Could not create the IOS images directory {}: {}".format(cls.getImageDirectory(), e))
            return

        compressed = False
        try:
            compressed = isIOSCompressed(path)
        except (OSError, ValueError):
            pass  # ignore errors if we cannot find out the IOS image is compressed.
        if compressed:
            reply = QtWidgets.QMessageBox.question(parent, "IOS image", "Would you like to decompress this IOS image?", QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if reply == QtWidgets.QMessageBox.Yes:
                decompressed_image_path = os.path.join(cls.getImageDirectory(), os.path.basename(os.path.splitext(path)[0] + ".image"))
                worker = DecompressIOSWorker(path, decompressed_image_path)
                progress_dialog = ProgressDialog(worker,
                                                 "IOS image",
                                                 "Decompressing IOS image {}...".format(os.path.basename(path)),
                                                 "Cancel", busy=True, parent=parent)
                progress_dialog.show()
                if progress_dialog.exec_() is not False:
                    path = decompressed_image_path

        path = ImageManager.instance().askCopyUploadImage(parent, path, server, "DYNAMIPS")

        return path

    @staticmethod
    def getMinimumRequiredRAM(path):
        """
        Returns the minimum RAM required to run an IOS image.

        :param path: path to the IOS image

        :returns: minimum RAM in MB or 0 if there is an error
        """

        try:
            if isIOSCompressed(path):
                zip_file = zipfile.ZipFile(path, "r")
                decompressed_size = 0
                for zip_info in zip_file.infolist():
                    decompressed_size += zip_info.file_size
            else:
                decompressed_size = os.path.getsize(path)
        except (zipfile.BadZipFile, OSError):
            return 0

        # get the size in MB
        decompressed_size = (decompressed_size / (1000 * 1000)) + 1
        # round up to the closest multiple of 32 (step of the RAM SpinBox)
        return math.ceil(decompressed_size / 32) * 32

    def _decompressIOSSlot(self):
        """
        Slot to decompress an IOS image.
        """

        item = self.uiIOSRoutersTreeWidget.currentItem()
        if item:
            key = item.data(0, QtCore.Qt.UserRole)
            ios_router = self._ios_routers[key]
            path = ios_router["image"]
            if not os.path.isfile(path):
                QtWidgets.QMessageBox.critical(self, "IOS image", "IOS image file {} is does not exist".format(path))
                return
            try:
                if not isIOSCompressed(path):
                    QtWidgets.QMessageBox.critical(self, "IOS image", "IOS image {} is not compressed".format(os.path.basename(path)))
                    return
            except OSError as e:
                # errno 22, invalid argument means the file system where the IOS image is located doesn't support mmap
                if e.errno == 22:
                    QtWidgets.QMessageBox.critical(self, "IOS image", "IOS image {} cannot be memory mapped, most likely because the file system doesn't support it".format(os.path.basename(path)))
                else:
                    QtWidgets.QMessageBox.critical(self, "IOS image", "Could not determine if the IOS image is compressed: {}".format(e))
                return
            except ValueError as e:
                QtWidgets.QMessageBox.critical(self, "IOS image", "Could not determine if the IOS image is compressed: {}".format(e))
                return

            decompressed_image_path = os.path.splitext(path)[0] + ".image"
            if os.path.isfile(decompressed_image_path):
                QtWidgets.QMessageBox.critical(self, "IOS image", "Decompressed IOS image {} already exist".format(os.path.basename(decompressed_image_path)))
                return

            worker = DecompressIOSWorker(path, decompressed_image_path)
            progress_dialog = ProgressDialog(worker,
                                             "IOS image",
                                             "Decompressing IOS image {}...".format(path),
                                             "Cancel", busy=True, parent=self)
            progress_dialog.show()
            if progress_dialog.exec_() is not False:
                ios_router["image"] = decompressed_image_path
                self._refreshInfo(ios_router)

    def _createSectionItem(self, name):

        section_item = QtWidgets.QTreeWidgetItem(self.uiIOSRouterInfoTreeWidget)
        section_item.setText(0, name)
        font = section_item.font(0)
        font.setBold(True)
        section_item.setFont(0, font)
        return section_item

    def _refreshInfo(self, ios_router):

        self.uiIOSRouterInfoTreeWidget.clear()

        # fill out the General section
        section_item = self._createSectionItem("General")
        QtWidgets.QTreeWidgetItem(section_item, ["Name:", ios_router["name"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Server:", ios_router["server"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Platform:", ios_router["platform"]])
        if ios_router["chassis"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Chassis:", ios_router["chassis"]])
        QtWidgets.QTreeWidgetItem(section_item, ["Image:", ios_router["image"]])
        if ios_router["idlepc"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Idle-PC:", ios_router["idlepc"]])
        if ios_router["startup_config"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Startup-config:", ios_router["startup_config"]])
        if ios_router["private_config"]:
            QtWidgets.QTreeWidgetItem(section_item, ["Private-config:", ios_router["private_config"]])
        if ios_router["platform"] == "c7200":
            QtWidgets.QTreeWidgetItem(section_item, ["Midplane:", ios_router["midplane"]])
            QtWidgets.QTreeWidgetItem(section_item, ["NPE:", ios_router["npe"]])

        # fill out the Memories and disk section
        section_item = self._createSectionItem("Memories and disks")
        QtWidgets.QTreeWidgetItem(section_item, ["RAM:", "{} MiB".format(ios_router["ram"])])
        QtWidgets.QTreeWidgetItem(section_item, ["NVRAM:", "{} KiB".format(ios_router["nvram"])])
        if "iomem" in ios_router and ios_router["iomem"]:
            QtWidgets.QTreeWidgetItem(section_item, ["I/O memory:", "{}%".format(ios_router["iomem"])])
        QtWidgets.QTreeWidgetItem(section_item, ["PCMCIA disk0:", "{} MiB".format(ios_router["disk0"])])
        QtWidgets.QTreeWidgetItem(section_item, ["PCMCIA disk1:", "{} MiB".format(ios_router["disk1"])])
        QtWidgets.QTreeWidgetItem(section_item, ["Auto delete:", "{}".format(ios_router["auto_delete_disks"])])

        # fill out the Adapters section
        section_item = self._createSectionItem("Adapters")
        for slot_id in range(0, 7):
            slot = "slot{}".format(slot_id)
            if slot in ios_router and ios_router[slot]:
                QtWidgets.QTreeWidgetItem(section_item, ["Slot {}:".format(slot_id), ios_router[slot]])
        if section_item.childCount() == 0:
            self.uiIOSRouterInfoTreeWidget.takeTopLevelItem(self.uiIOSRouterInfoTreeWidget.indexOfTopLevelItem(section_item))

        # fill out the WICs section
        section_item = self._createSectionItem("WICs")
        for wic_id in range(0, 3):
            wic = "wic{}".format(wic_id)
            if wic in ios_router and ios_router[wic]:
                QtWidgets.QTreeWidgetItem(section_item, ["WIC {}:".format(wic_id), ios_router[wic]])
        if section_item.childCount() == 0:
            self.uiIOSRouterInfoTreeWidget.takeTopLevelItem(self.uiIOSRouterInfoTreeWidget.indexOfTopLevelItem(section_item))

        self.uiIOSRouterInfoTreeWidget.expandAll()
        self.uiIOSRouterInfoTreeWidget.resizeColumnToContents(0)
        self.uiIOSRouterInfoTreeWidget.resizeColumnToContents(1)

    def loadPreferences(self):
        """
        Loads the IOS router preferences.
        """

        dynamips_module = Dynamips.instance()
        self._ios_routers = copy.deepcopy(dynamips_module.iosRouters())
        self._items.clear()

        for key, ios_router in self._ios_routers.items():
            item = QtWidgets.QTreeWidgetItem(self.uiIOSRoutersTreeWidget)
            item.setText(0, ios_router["name"])
            item.setIcon(0, QtGui.QIcon(ios_router["symbol"]))
            item.setData(0, QtCore.Qt.UserRole, key)
            self._items.append(item)

        if self._items:
            self.uiIOSRoutersTreeWidget.setCurrentItem(self._items[0])
            self.uiIOSRoutersTreeWidget.sortByColumn(0, QtCore.Qt.AscendingOrder)

    def savePreferences(self):
        """
        Saves the IOS router preferences.
        """

        Dynamips.instance().setIOSRouters(self._ios_routers)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import psutil

from gns3.qt import QtCore, QtWidgets, QtGui
from gns3.servers import Servers
from ..gns3_vm import GNS3VM
from ..dialogs.preferences_dialog import PreferencesDialog
from ..ui.setup_wizard_ui import Ui_SetupWizard
from ..utils.progress_dialog import ProgressDialog
from ..utils.wait_for_vm_worker import WaitForVMWorker
from ..utils.wait_for_connection_worker import WaitForConnectionWorker


class SetupWizard(QtWidgets.QWizard, Ui_SetupWizard):

    """
    Base class for VM wizard.
    """

    def __init__(self, parent):

        super().__init__(parent)
        self.setupUi(self)

        self.setWizardStyle(QtWidgets.QWizard.ModernStyle)
        if sys.platform.startswith("darwin"):
            # we want to see the cancel button on OSX
            self.setOptions(QtWidgets.QWizard.NoDefaultButton)

        self._server = Servers.instance().localServer()
        self.uiRefreshPushButton.clicked.connect(self._refreshVMListSlot)
        self.uiVmwareRadioButton.clicked.connect(self._listVMwareVMsSlot)
        self.uiVirtualBoxRadioButton.clicked.connect(self._listVirtualBoxVMsSlot)
        self.uiVMwareBannerButton.clicked.connect(self._VMwareBannerButtonClickedSlot)
        settings = parent.settings()
        self.uiShowCheckBox.setChecked(settings["hide_setup_wizard"])

        # by default all radio buttons are unchecked
        self.uiVmwareRadioButton.setAutoExclusive(False)
        self.uiVirtualBoxRadioButton.setAutoExclusive(False)
        self.uiVmwareRadioButton.setChecked(False)
        self.uiVirtualBoxRadioButton.setChecked(False)

        if sys.platform.startswith("darwin"):
            self.uiVMwareBannerButton.setIcon(QtGui.QIcon(":/images/vmware_fusion_banner.jpg"))
        else:
            self.uiVMwareBannerButton.setIcon(QtGui.QIcon(":/images/vmware_workstation_banner.jpg"))

    def _VMwareBannerButtonClickedSlot(self):
        if sys.platform.startswith("darwin"):
            url = "http://send.onenetworkdirect.net/z/616461/CD225091/"
        else:
            url = "http://send.onenetworkdirect.net/z/616460/CD225091/"
        QtGui.QDesktopServices.openUrl(QtCore.QUrl(url))

    def _listVMwareVMsSlot(self):
        """
        Slot to refresh the VMware VMs list.
        """

        self.uiVirtualBoxRadioButton.setChecked(False)
        from gns3.modules import VMware
        settings = VMware.instance().settings()
        if not os.path.exists(settings["vmrun_path"]):
            QtWidgets.QMessageBox.critical(self, "VMware", "VMware vmrun tool could not be found, VMware or the VIX API (required for VMware player) is probably not installed. You can download it from https://www.vmware.com/support/developer/vix-api/")
            return
        self._refreshVMListSlot()

    def _listVirtualBoxVMsSlot(self):
        """
        Slot to refresh the VirtualBox VMs list.
        """

        self.uiVmwareRadioButton.setChecked(False)
        from gns3.modules import VirtualBox
        settings = VirtualBox.instance().settings()
        if not os.path.exists(settings["vboxmanage_path"]):
            QtWidgets.QMessageBox.critical(self, "VirtualBox", "VBoxManage could not be found, VirtualBox is probably not installed")
            return
        self._refreshVMListSlot()

    def showit(self):
        """
        Either this dialog should be automatically showed at startup.

        :returns: boolean
        """

        return not self.uiShowCheckBox.isChecked()

    def _setPreferencesPane(self, dialog, name):
        """
        Finds the first child of the QTreeWidgetItem name.

        :param dialog: PreferencesDialog instance
        :param name: QTreeWidgetItem name

        :returns: current QWidget
        """

        pane = dialog.uiTreeWidget.findItems(name, QtCore.Qt.MatchFixedString)[0]
        child_pane = pane.child(0)
        dialog.uiTreeWidget.setCurrentItem(child_pane)
        return dialog.uiStackedWidget.currentWidget()

    def initializePage(self, page_id):
        """
        Initialize Wizard pages.

        :param page_id: page identifier
        """

        super().initializePage(page_id)
        if self.page(page_id) == self.uiVMWizardPage:
            cpu_count = psutil.cpu_count()
            self.uiCPUSpinBox.setValue(cpu_count)
            # we want to allocate half of the available physical memory
            ram = int(psutil.virtual_memory().total / (1024 * 1024) / 2)
            # value must be a multiple of 4 (VMware requirement)
            ram -= ram % 4
            self.uiRAMSpinBox.setValue(ram)

    def validateCurrentPage(self):
        """
        Validates the settings.
        """

        gns3_vm = GNS3VM.instance()
        servers = Servers.instance()
        if self.currentPage() == self.uiVMWizardPage:
            vmname = self.uiVMListComboBox.currentText()
            if vmname:
                # save the GNS3 VM settings
                vm_settings = {"auto_start": True,
                               "vmname": vmname,
                               "vmx_path": self.uiVMListComboBox.currentData()}
                if self.uiVmwareRadioButton.isChecked():
                    vm_settings["virtualization"] = "VMware"
                elif self.uiVirtualBoxRadioButton.isChecked():
                    vm_settings["virtualization"] = "VirtualBox"
                gns3_vm.setSettings(vm_settings)
                servers.save()

                # set the vCPU count and RAM
                vpcus = self.uiCPUSpinBox.value()
                ram = self.uiRAMSpinBox.value()
                if ram < 1024:
                    QtWidgets.QMessageBox.warning(self, "GNS3 VM memory", "It is recommended to allocate a minimum of 1024 MB of RAM to the GNS3 VM")
                available_ram = int(psutil.virtual_memory().available / (1024 * 1024))
                if ram > available_ram:
                    QtWidgets.QMessageBox.warning(self, "GNS3 VM memory", "You have probably allocated too much memory for the GNS3 VM! (available memory is {} MB)".format(available_ram))
                if gns3_vm.setvCPUandRAM(vpcus, ram) is False:
                    QtWidgets.QMessageBox.warning(self, "GNS3 VM", "Could not configure vCPUs and RAM amounts for the GNS3 VM")

                # start the GNS3 VM
                servers.initVMServer()
                worker = WaitForVMWorker()
                progress_dialog = ProgressDialog(worker, "GNS3 VM", "Starting the GNS3 VM...", "Cancel", busy=True, parent=self)
                progress_dialog.show()
                if progress_dialog.exec_():
                    previous_local_server_ip = servers.localServer().host()
                    new_local_server_ip = gns3_vm.adjustLocalServerIP()
                    self.uiShowCheckBox.setChecked(True)
                    # restart the local server if necessary
                    if new_local_server_ip != previous_local_server_ip:
                        servers.stopLocalServer(wait=True)
                        if servers.startLocalServer():
                            worker = WaitForConnectionWorker(new_local_server_ip, servers.localServer().port())
                            dialog = ProgressDialog(worker, "Local server", "Connecting...", "Cancel", busy=True, parent=self)
                            dialog.show()
                            dialog.exec_()
            else:
                if not self.uiVmwareRadioButton.isChecked() and not self.uiVirtualBoxRadioButton.isChecked():
                    QtWidgets.QMessageBox.warning(self, "GNS3 VM", "Please select VMware or VirtualBox")
                else:
                    QtWidgets.QMessageBox.warning(self, "GNS3 VM", "Please select a VM. If no VM is listed, check if the GNS3 VM is correctly imported and press refresh.")
                return False
        elif self.currentPage() == self.uiAddVMsWizardPage:

            use_local_server = self.uiLocalRadioButton.isChecked()
            if use_local_server:
                # deactivate the GNS3 VM if using the local server
                vm_settings = {"auto_start": False}
                gns3_vm.setSettings(vm_settings)
                servers.save()
                self.uiShowCheckBox.setChecked(True)

            from gns3.modules import Dynamips
            Dynamips.instance().setSettings({"use_local_server": use_local_server})
            if sys.platform.startswith("linux"):
                # IOU only works on Linux
                from gns3.modules import IOU
                IOU.instance().setSettings({"use_local_server": use_local_server})
            from gns3.modules import Qemu
            Qemu.instance().setSettings({"use_local_server": use_local_server})
            from gns3.modules import VPCS
            VPCS.instance().setSettings({"use_local_server": use_local_server})

            dialog = PreferencesDialog(self)
            if self.uiAddIOSRouterCheckBox.isChecked():
                self._setPreferencesPane(dialog, "Dynamips").uiNewIOSRouterPushButton.clicked.emit(False)
            if self.uiAddIOUDeviceCheckBox.isChecked():
                self._setPreferencesPane(dialog, "IOS on UNIX").uiNewIOUDevicePushButton.clicked.emit(False)
            if self.uiAddQemuVMcheckBox.isChecked():
                self._setPreferencesPane(dialog, "QEMU").uiNewQemuVMPushButton.clicked.emit(False)
            if self.uiAddVirtualBoxVMcheckBox.isChecked():
                self._setPreferencesPane(dialog, "VirtualBox").uiNewVirtualBoxVMPushButton.clicked.emit(False)
            if self.uiAddVMwareVMcheckBox.isChecked():
                self._setPreferencesPane(dialog, "VMware").uiNewVMwareVMPushButton.clicked.emit(False)
            dialog.exec_()
        return True

    def _refreshVMListSlot(self):
        """
        Refresh the list of VM available in VMware or VirtualBox.
        """

        server = Servers.instance().localServer()
        if self.uiVmwareRadioButton.isChecked():
            server.get("/vmware/vms", self._getVMsFromServerCallback)
        elif self.uiVirtualBoxRadioButton.isChecked():
            server.get("/virtualbox/vms", self._getVMsFromServerCallback)

    def _getVMsFromServerCallback(self, result, error=False, **kwargs):
        """
        Callback for getVMsFromServer.

        :param progress_dialog: QProgressDialog instance
        :param result: server response
        :param error: indicates an error (boolean)
        """

        if error:
            QtWidgets.QMessageBox.critical(self, "VM List", "{}".format(result["message"]))
        else:
            self.uiVMListComboBox.clear()
            for vm in result:
                self.uiVMListComboBox.addItem(vm["vmname"], vm.get("vmx_path", ""))
            gns3_vm = Servers.instance().vmSettings()
            index = self.uiVMListComboBox.findText(gns3_vm["vmname"])
            if index != -1:
                self.uiVMListComboBox.setCurrentIndex(index)
            else:
                index = self.uiVMListComboBox.findText("GNS3 VM")
                if index != -1:
                    self.uiVMListComboBox.setCurrentIndex(index)
                else:
                    QtWidgets.QMessageBox.critical(self, "GNS3 VM", "Could not find a VM named 'GNS3 VM', is it imported in VMware or VirtualBox?")

    def done(self, result):
        """
        This dialog is closed.

        :param result: ignored
        """

        settings = self.parentWidget().settings()
        settings["hide_setup_wizard"] = self.uiShowCheckBox.isChecked()
        self.parentWidget().setSettings(settings)
        super().done(result)

    def nextId(self):
        """
        Wizard rules!
        """

        current_id = self.currentId()
        if self.page(current_id) == self.uiServerWizardPage and not self.uiVMRadioButton.isChecked():
            # skip the GNS3 VM page if using the local server.
            return self.uiServerWizardPage.nextId() + 1
        return QtWidgets.QWizard.nextId(self)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/dynamips/ui/ios_router_configuration_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_iosRouterConfigPageWidget(object):

    def setupUi(self, iosRouterConfigPageWidget):
        iosRouterConfigPageWidget.setObjectName("iosRouterConfigPageWidget")
        iosRouterConfigPageWidget.resize(476, 510)
        self.vboxlayout = QtWidgets.QVBoxLayout(iosRouterConfigPageWidget)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiTabWidget = QtWidgets.QTabWidget(iosRouterConfigPageWidget)
        self.uiTabWidget.setObjectName("uiTabWidget")
        self.uiGeneralPageWidget = QtWidgets.QWidget()
        self.uiGeneralPageWidget.setObjectName("uiGeneralPageWidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.uiGeneralPageWidget)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiNameLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout_2.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout_2.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        self.uiPlatformLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformLabel.setObjectName("uiPlatformLabel")
        self.gridLayout_2.addWidget(self.uiPlatformLabel, 3, 0, 1, 1)
        self.uiPlatformTextLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPlatformTextLabel.setText("")
        self.uiPlatformTextLabel.setObjectName("uiPlatformTextLabel")
        self.gridLayout_2.addWidget(self.uiPlatformTextLabel, 3, 1, 1, 1)
        self.uiChassisLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiChassisLabel.setObjectName("uiChassisLabel")
        self.gridLayout_2.addWidget(self.uiChassisLabel, 4, 0, 1, 1)
        self.uiChassisTextLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiChassisTextLabel.setText("")
        self.uiChassisTextLabel.setObjectName("uiChassisTextLabel")
        self.gridLayout_2.addWidget(self.uiChassisTextLabel, 4, 1, 1, 1)
        self.uiIOSImageLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiIOSImageLabel.setObjectName("uiIOSImageLabel")
        self.gridLayout_2.addWidget(self.uiIOSImageLabel, 8, 0, 1, 1)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiIOSImageLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiIOSImageLineEdit.setObjectName("uiIOSImageLineEdit")
        self.horizontalLayout_5.addWidget(self.uiIOSImageLineEdit)
        self.uiIOSImageToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiIOSImageToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiIOSImageToolButton.setObjectName("uiIOSImageToolButton")
        self.horizontalLayout_5.addWidget(self.uiIOSImageToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_5, 8, 1, 1, 1)
        self.uiStartupConfigLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiStartupConfigLabel.setObjectName("uiStartupConfigLabel")
        self.gridLayout_2.addWidget(self.uiStartupConfigLabel, 9, 0, 1, 1)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiStartupConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiStartupConfigLineEdit.setObjectName("uiStartupConfigLineEdit")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigLineEdit)
        self.uiStartupConfigToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiStartupConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiStartupConfigToolButton.setObjectName("uiStartupConfigToolButton")
        self.horizontalLayout_4.addWidget(self.uiStartupConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_4, 9, 1, 1, 1)
        self.uiPrivateConfigLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiPrivateConfigLabel.setObjectName("uiPrivateConfigLabel")
        self.gridLayout_2.addWidget(self.uiPrivateConfigLabel, 10, 0, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.uiPrivateConfigLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiPrivateConfigLineEdit.setObjectName("uiPrivateConfigLineEdit")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigLineEdit)
        self.uiPrivateConfigToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiPrivateConfigToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiPrivateConfigToolButton.setObjectName("uiPrivateConfigToolButton")
        self.horizontalLayout_6.addWidget(self.uiPrivateConfigToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 10, 1, 1, 1)
        self.uiConsolePortLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiConsolePortLabel.setObjectName("uiConsolePortLabel")
        self.gridLayout_2.addWidget(self.uiConsolePortLabel, 13, 0, 1, 1)
        self.uiConsolePortSpinBox = QtWidgets.QSpinBox(self.uiGeneralPageWidget)
        self.uiConsolePortSpinBox.setMaximum(65535)
        self.uiConsolePortSpinBox.setObjectName("uiConsolePortSpinBox")
        self.gridLayout_2.addWidget(self.uiConsolePortSpinBox, 13, 1, 1, 1)
        self.uiAuxPortLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiAuxPortLabel.setObjectName("uiAuxPortLabel")
        self.gridLayout_2.addWidget(self.uiAuxPortLabel, 14, 0, 1, 1)
        self.uiAuxPortSpinBox = QtWidgets.QSpinBox(self.uiGeneralPageWidget)
        self.uiAuxPortSpinBox.setMaximum(65535)
        self.uiAuxPortSpinBox.setObjectName("uiAuxPortSpinBox")
        self.gridLayout_2.addWidget(self.uiAuxPortSpinBox, 14, 1, 1, 1)
        self.uiMidplaneLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiMidplaneLabel.setObjectName("uiMidplaneLabel")
        self.gridLayout_2.addWidget(self.uiMidplaneLabel, 15, 0, 1, 1)
        self.uiMidplaneComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiMidplaneComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiMidplaneComboBox.sizePolicy().hasHeightForWidth())
        self.uiMidplaneComboBox.setSizePolicy(sizePolicy)
        self.uiMidplaneComboBox.setObjectName("uiMidplaneComboBox")
        self.gridLayout_2.addWidget(self.uiMidplaneComboBox, 15, 1, 1, 1)
        self.uiNPELabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiNPELabel.setObjectName("uiNPELabel")
        self.gridLayout_2.addWidget(self.uiNPELabel, 16, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(263, 151, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem, 17, 1, 1, 1)
        self.uiNPEComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiNPEComboBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNPEComboBox.sizePolicy().hasHeightForWidth())
        self.uiNPEComboBox.setSizePolicy(sizePolicy)
        self.uiNPEComboBox.setObjectName("uiNPEComboBox")
        self.gridLayout_2.addWidget(self.uiNPEComboBox, 16, 1, 1, 1)
        self.uiSymbolLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiSymbolLabel.setObjectName("uiSymbolLabel")
        self.gridLayout_2.addWidget(self.uiSymbolLabel, 5, 0, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiSymbolLineEdit = QtWidgets.QLineEdit(self.uiGeneralPageWidget)
        self.uiSymbolLineEdit.setObjectName("uiSymbolLineEdit")
        self.horizontalLayout_7.addWidget(self.uiSymbolLineEdit)
        self.uiSymbolToolButton = QtWidgets.QToolButton(self.uiGeneralPageWidget)
        self.uiSymbolToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiSymbolToolButton.setObjectName("uiSymbolToolButton")
        self.horizontalLayout_7.addWidget(self.uiSymbolToolButton)
        self.gridLayout_2.addLayout(self.horizontalLayout_7, 5, 1, 1, 1)
        self.uiCategoryLabel = QtWidgets.QLabel(self.uiGeneralPageWidget)
        self.uiCategoryLabel.setObjectName("uiCategoryLabel")
        self.gridLayout_2.addWidget(self.uiCategoryLabel, 6, 0, 1, 1)
        self.uiCategoryComboBox = QtWidgets.QComboBox(self.uiGeneralPageWidget)
        self.uiCategoryComboBox.setObjectName("uiCategoryComboBox")
        self.gridLayout_2.addWidget(self.uiCategoryComboBox, 6, 1, 1, 1)
        self.uiTabWidget.addTab(self.uiGeneralPageWidget, "")
        self.uiMemoriesPageWidget = QtWidgets.QWidget()
        self.uiMemoriesPageWidget.setObjectName("uiMemoriesPageWidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.uiMemoriesPageWidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.uiRamLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiRamLabel.setObjectName("uiRamLabel")
        self.gridLayout_5.addWidget(self.uiRamLabel, 0, 0, 1, 1)
        self.uiRamSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRamSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRamSpinBox.setSizePolicy(sizePolicy)
        self.uiRamSpinBox.setMinimum(32)
        self.uiRamSpinBox.setMaximum(65535)
        self.uiRamSpinBox.setSingleStep(32)
        self.uiRamSpinBox.setProperty("value", 128)
        self.uiRamSpinBox.setObjectName("uiRamSpinBox")
        self.gridLayout_5.addWidget(self.uiRamSpinBox, 0, 1, 1, 1)
        self.uiNvramLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiNvramLabel.setObjectName("uiNvramLabel")
        self.gridLayout_5.addWidget(self.uiNvramLabel, 1, 0, 1, 1)
        self.uiNvramSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNvramSpinBox.sizePolicy().hasHeightForWidth())
        self.uiNvramSpinBox.setSizePolicy(sizePolicy)
        self.uiNvramSpinBox.setMinimum(32)
        self.uiNvramSpinBox.setMaximum(65535)
        self.uiNvramSpinBox.setSingleStep(32)
        self.uiNvramSpinBox.setProperty("value", 128)
        self.uiNvramSpinBox.setObjectName("uiNvramSpinBox")
        self.gridLayout_5.addWidget(self.uiNvramSpinBox, 1, 1, 1, 1)
        self.uiIomemLabel = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiIomemLabel.setObjectName("uiIomemLabel")
        self.gridLayout_5.addWidget(self.uiIomemLabel, 2, 0, 1, 1)
        self.uiIomemSpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        self.uiIomemSpinBox.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiIomemSpinBox.sizePolicy().hasHeightForWidth())
        self.uiIomemSpinBox.setSizePolicy(sizePolicy)
        self.uiIomemSpinBox.setMaximum(100)
        self.uiIomemSpinBox.setSingleStep(5)
        self.uiIomemSpinBox.setProperty("value", 5)
        self.uiIomemSpinBox.setObjectName("uiIomemSpinBox")
        self.gridLayout_5.addWidget(self.uiIomemSpinBox, 2, 1, 1, 1)
        self.uiDisk0Label = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiDisk0Label.setObjectName("uiDisk0Label")
        self.gridLayout_5.addWidget(self.uiDisk0Label, 3, 0, 1, 1)
        self.uiDisk0SpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk0SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk0SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk0SpinBox.setMaximum(99999)
        self.uiDisk0SpinBox.setSingleStep(4)
        self.uiDisk0SpinBox.setObjectName("uiDisk0SpinBox")
        self.gridLayout_5.addWidget(self.uiDisk0SpinBox, 3, 1, 1, 1)
        self.uiDisk1Label = QtWidgets.QLabel(self.uiMemoriesPageWidget)
        self.uiDisk1Label.setObjectName("uiDisk1Label")
        self.gridLayout_5.addWidget(self.uiDisk1Label, 4, 0, 1, 1)
        self.uiDisk1SpinBox = QtWidgets.QSpinBox(self.uiMemoriesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDisk1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiDisk1SpinBox.setSizePolicy(sizePolicy)
        self.uiDisk1SpinBox.setMaximum(99999)
        self.uiDisk1SpinBox.setSingleStep(4)
        self.uiDisk1SpinBox.setObjectName("uiDisk1SpinBox")
        self.gridLayout_5.addWidget(self.uiDisk1SpinBox, 4, 1, 1, 1)
        self.uiAutoDeleteCheckBox = QtWidgets.QCheckBox(self.uiMemoriesPageWidget)
        self.uiAutoDeleteCheckBox.setObjectName("uiAutoDeleteCheckBox")
        self.gridLayout_5.addWidget(self.uiAutoDeleteCheckBox, 5, 0, 1, 2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 21, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem1, 6, 0, 1, 1)
        self.uiTabWidget.addTab(self.uiMemoriesPageWidget, "")
        self.uiSlotsPageWidget = QtWidgets.QWidget()
        self.uiSlotsPageWidget.setObjectName("uiSlotsPageWidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiSlotsPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiAdaptersGroupBox = QtWidgets.QGroupBox(self.uiSlotsPageWidget)
        self.uiAdaptersGroupBox.setObjectName("uiAdaptersGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiAdaptersGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.uiSlot1comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot1comboBox.setObjectName("uiSlot1comboBox")
        self.gridLayout.addWidget(self.uiSlot1comboBox, 1, 1, 1, 1)
        self.uiSlot0Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot0Label.sizePolicy().hasHeightForWidth())
        self.uiSlot0Label.setSizePolicy(sizePolicy)
        self.uiSlot0Label.setObjectName("uiSlot0Label")
        self.gridLayout.addWidget(self.uiSlot0Label, 0, 0, 1, 1)
        self.uiSlot0comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot0comboBox.setObjectName("uiSlot0comboBox")
        self.gridLayout.addWidget(self.uiSlot0comboBox, 0, 1, 1, 1)
        self.uiSlot1Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot1Label.sizePolicy().hasHeightForWidth())
        self.uiSlot1Label.setSizePolicy(sizePolicy)
        self.uiSlot1Label.setObjectName("uiSlot1Label")
        self.gridLayout.addWidget(self.uiSlot1Label, 1, 0, 1, 1)
        self.uiSlot2Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot2Label.sizePolicy().hasHeightForWidth())
        self.uiSlot2Label.setSizePolicy(sizePolicy)
        self.uiSlot2Label.setObjectName("uiSlot2Label")
        self.gridLayout.addWidget(self.uiSlot2Label, 2, 0, 1, 1)
        self.uiSlot2comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot2comboBox.setObjectName("uiSlot2comboBox")
        self.gridLayout.addWidget(self.uiSlot2comboBox, 2, 1, 1, 1)
        self.uiSlot3Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot3Label.sizePolicy().hasHeightForWidth())
        self.uiSlot3Label.setSizePolicy(sizePolicy)
        self.uiSlot3Label.setObjectName("uiSlot3Label")
        self.gridLayout.addWidget(self.uiSlot3Label, 3, 0, 1, 1)
        self.uiSlot3comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot3comboBox.setObjectName("uiSlot3comboBox")
        self.gridLayout.addWidget(self.uiSlot3comboBox, 3, 1, 1, 1)
        self.uiSlot4Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot4Label.sizePolicy().hasHeightForWidth())
        self.uiSlot4Label.setSizePolicy(sizePolicy)
        self.uiSlot4Label.setObjectName("uiSlot4Label")
        self.gridLayout.addWidget(self.uiSlot4Label, 4, 0, 1, 1)
        self.uiSlot4comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot4comboBox.setObjectName("uiSlot4comboBox")
        self.gridLayout.addWidget(self.uiSlot4comboBox, 4, 1, 1, 1)
        self.uiSlot5Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot5Label.sizePolicy().hasHeightForWidth())
        self.uiSlot5Label.setSizePolicy(sizePolicy)
        self.uiSlot5Label.setObjectName("uiSlot5Label")
        self.gridLayout.addWidget(self.uiSlot5Label, 5, 0, 1, 1)
        self.uiSlot5comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot5comboBox.setObjectName("uiSlot5comboBox")
        self.gridLayout.addWidget(self.uiSlot5comboBox, 5, 1, 1, 1)
        self.uiSlot6Label = QtWidgets.QLabel(self.uiAdaptersGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSlot6Label.sizePolicy().hasHeightForWidth())
        self.uiSlot6Label.setSizePolicy(sizePolicy)
        self.uiSlot6Label.setObjectName("uiSlot6Label")
        self.gridLayout.addWidget(self.uiSlot6Label, 6, 0, 1, 1)
        self.uiSlot6comboBox = QtWidgets.QComboBox(self.uiAdaptersGroupBox)
        self.uiSlot6comboBox.setObjectName("uiSlot6comboBox")
        self.gridLayout.addWidget(self.uiSlot6comboBox, 6, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiAdaptersGroupBox)
        self.uiWicsGroupBox = QtWidgets.QGroupBox(self.uiSlotsPageWidget)
        self.uiWicsGroupBox.setObjectName("uiWicsGroupBox")
        self.gridlayout = QtWidgets.QGridLayout(self.uiWicsGroupBox)
        self.gridlayout.setObjectName("gridlayout")
        self.uiWic0Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic0Label.sizePolicy().hasHeightForWidth())
        self.uiWic0Label.setSizePolicy(sizePolicy)
        self.uiWic0Label.setObjectName("uiWic0Label")
        self.gridlayout.addWidget(self.uiWic0Label, 0, 0, 1, 1)
        self.uiWic0comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic0comboBox.setObjectName("uiWic0comboBox")
        self.gridlayout.addWidget(self.uiWic0comboBox, 0, 1, 1, 1)
        self.uiWic1Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic1Label.sizePolicy().hasHeightForWidth())
        self.uiWic1Label.setSizePolicy(sizePolicy)
        self.uiWic1Label.setObjectName("uiWic1Label")
        self.gridlayout.addWidget(self.uiWic1Label, 1, 0, 1, 1)
        self.uiWic1comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic1comboBox.setObjectName("uiWic1comboBox")
        self.gridlayout.addWidget(self.uiWic1comboBox, 1, 1, 1, 1)
        self.uiWic2Label = QtWidgets.QLabel(self.uiWicsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiWic2Label.sizePolicy().hasHeightForWidth())
        self.uiWic2Label.setSizePolicy(sizePolicy)
        self.uiWic2Label.setObjectName("uiWic2Label")
        self.gridlayout.addWidget(self.uiWic2Label, 2, 0, 1, 1)
        self.uiWic2comboBox = QtWidgets.QComboBox(self.uiWicsGroupBox)
        self.uiWic2comboBox.setObjectName("uiWic2comboBox")
        self.gridlayout.addWidget(self.uiWic2comboBox, 2, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiWicsGroupBox)
        spacerItem2 = QtWidgets.QSpacerItem(325, 31, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem2)
        self.uiTabWidget.addTab(self.uiSlotsPageWidget, "")
        self.uiAdvancedPageWidget = QtWidgets.QWidget()
        self.uiAdvancedPageWidget.setObjectName("uiAdvancedPageWidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiAdvancedPageWidget)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiSystemGroupBox = QtWidgets.QGroupBox(self.uiAdvancedPageWidget)
        self.uiSystemGroupBox.setObjectName("uiSystemGroupBox")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.uiSystemGroupBox)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.uiSystemIdLineEdit = QtWidgets.QLineEdit(self.uiSystemGroupBox)
        self.uiSystemIdLineEdit.setObjectName("uiSystemIdLineEdit")
        self.gridLayout_6.addWidget(self.uiSystemIdLineEdit, 0, 1, 1, 1)
        self.label = QtWidgets.QLabel(self.uiSystemGroupBox)
        self.label.setObjectName("label")
        self.gridLayout_6.addWidget(self.label, 0, 0, 1, 1)
        self.uiBaseMacLabel = QtWidgets.QLabel(self.uiSystemGroupBox)
        self.uiBaseMacLabel.setObjectName("uiBaseMacLabel")
        self.gridLayout_6.addWidget(self.uiBaseMacLabel, 1, 0, 1, 1)
        self.uiBaseMACLineEdit = QtWidgets.QLineEdit(self.uiSystemGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiBaseMACLineEdit.sizePolicy().hasHeightForWidth())
        self.uiBaseMACLineEdit.setSizePolicy(sizePolicy)
        self.uiBaseMACLineEdit.setText("")
        self.uiBaseMACLineEdit.setObjectName("uiBaseMACLineEdit")
        self.gridLayout_6.addWidget(self.uiBaseMACLineEdit, 1, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiSystemGroupBox)
        self.uiOptimizationsGroupBox = QtWidgets.QGroupBox(self.uiAdvancedPageWidget)
        self.uiOptimizationsGroupBox.setObjectName("uiOptimizationsGroupBox")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.uiOptimizationsGroupBox)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.uiExecAreaLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiExecAreaLabel.setObjectName("uiExecAreaLabel")
        self.gridLayout_4.addWidget(self.uiExecAreaLabel, 3, 0, 1, 1)
        self.uiExecAreaSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiExecAreaSpinBox.sizePolicy().hasHeightForWidth())
        self.uiExecAreaSpinBox.setSizePolicy(sizePolicy)
        self.uiExecAreaSpinBox.setMaximum(4096)
        self.uiExecAreaSpinBox.setSingleStep(4)
        self.uiExecAreaSpinBox.setProperty("value", 64)
        self.uiExecAreaSpinBox.setObjectName("uiExecAreaSpinBox")
        self.gridLayout_4.addWidget(self.uiExecAreaSpinBox, 3, 1, 1, 1)
        self.uiSparseMemoryCheckBox = QtWidgets.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiSparseMemoryCheckBox.setChecked(False)
        self.uiSparseMemoryCheckBox.setObjectName("uiSparseMemoryCheckBox")
        self.gridLayout_4.addWidget(self.uiSparseMemoryCheckBox, 5, 0, 1, 2)
        self.uiIdlepcLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlepcLabel.setObjectName("uiIdlepcLabel")
        self.gridLayout_4.addWidget(self.uiIdlepcLabel, 0, 0, 1, 1)
        self.uiIdlepcLineEdit = QtWidgets.QLineEdit(self.uiOptimizationsGroupBox)
        self.uiIdlepcLineEdit.setObjectName("uiIdlepcLineEdit")
        self.gridLayout_4.addWidget(self.uiIdlepcLineEdit, 0, 1, 1, 1)
        self.uiIdlemaxLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlemaxLabel.setObjectName("uiIdlemaxLabel")
        self.gridLayout_4.addWidget(self.uiIdlemaxLabel, 1, 0, 1, 1)
        self.uiIdlemaxSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlemaxSpinBox.setMinimum(100)
        self.uiIdlemaxSpinBox.setMaximum(10000)
        self.uiIdlemaxSpinBox.setSingleStep(100)
        self.uiIdlemaxSpinBox.setProperty("value", 500)
        self.uiIdlemaxSpinBox.setObjectName("uiIdlemaxSpinBox")
        self.gridLayout_4.addWidget(self.uiIdlemaxSpinBox, 1, 1, 1, 1)
        self.uiIdlesleepLabel = QtWidgets.QLabel(self.uiOptimizationsGroupBox)
        self.uiIdlesleepLabel.setObjectName("uiIdlesleepLabel")
        self.gridLayout_4.addWidget(self.uiIdlesleepLabel, 2, 0, 1, 1)
        self.uiMmapCheckBox = QtWidgets.QCheckBox(self.uiOptimizationsGroupBox)
        self.uiMmapCheckBox.setChecked(True)
        self.uiMmapCheckBox.setObjectName("uiMmapCheckBox")
        self.gridLayout_4.addWidget(self.uiMmapCheckBox, 4, 0, 1, 2)
        self.uiIdlesleepSpinBox = QtWidgets.QSpinBox(self.uiOptimizationsGroupBox)
        self.uiIdlesleepSpinBox.setMinimum(1)
        self.uiIdlesleepSpinBox.setMaximum(500)
        self.uiIdlesleepSpinBox.setSingleStep(10)
        self.uiIdlesleepSpinBox.setProperty("value", 30)
        self.uiIdlesleepSpinBox.setObjectName("uiIdlesleepSpinBox")
        self.gridLayout_4.addWidget(self.uiIdlesleepSpinBox, 2, 1, 1, 1)
        self.verticalLayout_4.addWidget(self.uiOptimizationsGroupBox)
        spacerItem3 = QtWidgets.QSpacerItem(304, 251, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem3)
        self.uiTabWidget.addTab(self.uiAdvancedPageWidget, "")
        self.uiEnvironmentPageWidget = QtWidgets.QWidget()
        self.uiEnvironmentPageWidget.setObjectName("uiEnvironmentPageWidget")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiEnvironmentPageWidget)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiPowerSuppliesGroupBox = QtWidgets.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiPowerSuppliesGroupBox.setObjectName("uiPowerSuppliesGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiPowerSuppliesGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiPowerSupply2Label = QtWidgets.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply2Label.setObjectName("uiPowerSupply2Label")
        self.gridLayout_7.addWidget(self.uiPowerSupply2Label, 1, 0, 1, 1)
        self.uiPowerSupply1ComboBox = QtWidgets.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply1ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply1ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply1ComboBox.setObjectName("uiPowerSupply1ComboBox")
        self.uiPowerSupply1ComboBox.addItem("")
        self.uiPowerSupply1ComboBox.addItem("")
        self.gridLayout_7.addWidget(self.uiPowerSupply1ComboBox, 0, 1, 1, 1)
        self.uiPowerSupply1Label = QtWidgets.QLabel(self.uiPowerSuppliesGroupBox)
        self.uiPowerSupply1Label.setObjectName("uiPowerSupply1Label")
        self.gridLayout_7.addWidget(self.uiPowerSupply1Label, 0, 0, 1, 1)
        self.uiPowerSupply2ComboBox = QtWidgets.QComboBox(self.uiPowerSuppliesGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiPowerSupply2ComboBox.sizePolicy().hasHeightForWidth())
        self.uiPowerSupply2ComboBox.setSizePolicy(sizePolicy)
        self.uiPowerSupply2ComboBox.setObjectName("uiPowerSupply2ComboBox")
        self.uiPowerSupply2ComboBox.addItem("")
        self.uiPowerSupply2ComboBox.addItem("")
        self.gridLayout_7.addWidget(self.uiPowerSupply2ComboBox, 1, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiPowerSuppliesGroupBox)
        self.uiSensorsGroupBox = QtWidgets.QGroupBox(self.uiEnvironmentPageWidget)
        self.uiSensorsGroupBox.setObjectName("uiSensorsGroupBox")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.uiSensorsGroupBox)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.uiSensor1Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor1Label.setObjectName("uiSensor1Label")
        self.gridLayout_3.addWidget(self.uiSensor1Label, 0, 0, 1, 1)
        self.uiSensor1SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor1SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor1SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor1SpinBox.setMaximum(100)
        self.uiSensor1SpinBox.setObjectName("uiSensor1SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor1SpinBox, 0, 1, 1, 1)
        self.uiSensor2Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor2Label.setObjectName("uiSensor2Label")
        self.gridLayout_3.addWidget(self.uiSensor2Label, 1, 0, 1, 1)
        self.uiSensor2SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor2SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor2SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor2SpinBox.setMaximum(100)
        self.uiSensor2SpinBox.setObjectName("uiSensor2SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor2SpinBox, 1, 1, 1, 1)
        self.uiSensor3Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor3Label.setObjectName("uiSensor3Label")
        self.gridLayout_3.addWidget(self.uiSensor3Label, 2, 0, 1, 1)
        self.uiSensor3SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor3SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor3SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor3SpinBox.setMaximum(100)
        self.uiSensor3SpinBox.setObjectName("uiSensor3SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor3SpinBox, 2, 1, 1, 1)
        self.uiSensor4Label = QtWidgets.QLabel(self.uiSensorsGroupBox)
        self.uiSensor4Label.setObjectName("uiSensor4Label")
        self.gridLayout_3.addWidget(self.uiSensor4Label, 3, 0, 1, 1)
        self.uiSensor4SpinBox = QtWidgets.QSpinBox(self.uiSensorsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSensor4SpinBox.sizePolicy().hasHeightForWidth())
        self.uiSensor4SpinBox.setSizePolicy(sizePolicy)
        self.uiSensor4SpinBox.setMaximum(100)
        self.uiSensor4SpinBox.setObjectName("uiSensor4SpinBox")
        self.gridLayout_3.addWidget(self.uiSensor4SpinBox, 3, 1, 1, 1)
        self.verticalLayout_3.addWidget(self.uiSensorsGroupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 194, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem4)
        self.uiTabWidget.addTab(self.uiEnvironmentPageWidget, "")
        self.vboxlayout.addWidget(self.uiTabWidget)

        self.retranslateUi(iosRouterConfigPageWidget)
        self.uiTabWidget.setCurrentIndex(0)
        self.uiPowerSupply1ComboBox.setCurrentIndex(1)
        self.uiPowerSupply2ComboBox.setCurrentIndex(1)
        QtCore.QMetaObject.connectSlotsByName(iosRouterConfigPageWidget)
        iosRouterConfigPageWidget.setTabOrder(self.uiMidplaneComboBox, self.uiNPEComboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiNPEComboBox, self.uiSlot0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot0comboBox, self.uiSlot1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot1comboBox, self.uiSlot2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot2comboBox, self.uiSlot3comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot3comboBox, self.uiSlot4comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot4comboBox, self.uiSlot5comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot5comboBox, self.uiSlot6comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiSlot6comboBox, self.uiWic0comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic0comboBox, self.uiWic1comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic1comboBox, self.uiWic2comboBox)
        iosRouterConfigPageWidget.setTabOrder(self.uiWic2comboBox, self.uiBaseMACLineEdit)
        iosRouterConfigPageWidget.setTabOrder(self.uiBaseMACLineEdit, self.uiExecAreaSpinBox)

    def retranslateUi(self, iosRouterConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        iosRouterConfigPageWidget.setWindowTitle(_translate("iosRouterConfigPageWidget", "Dynamips IOS Router configuration"))
        self.uiNameLabel.setText(_translate("iosRouterConfigPageWidget", "Name:"))
        self.uiPlatformLabel.setText(_translate("iosRouterConfigPageWidget", "Platform:"))
        self.uiChassisLabel.setText(_translate("iosRouterConfigPageWidget", "Chassis:"))
        self.uiIOSImageLabel.setText(_translate("iosRouterConfigPageWidget", "IOS image path:"))
        self.uiIOSImageToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiStartupConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial startup-config:"))
        self.uiStartupConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiPrivateConfigLabel.setText(_translate("iosRouterConfigPageWidget", "Initial private-config:"))
        self.uiPrivateConfigToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiConsolePortLabel.setText(_translate("iosRouterConfigPageWidget", "Console port:"))
        self.uiAuxPortLabel.setText(_translate("iosRouterConfigPageWidget", "Aux port:"))
        self.uiMidplaneLabel.setText(_translate("iosRouterConfigPageWidget", "Midplane:"))
        self.uiNPELabel.setText(_translate("iosRouterConfigPageWidget", "NPE:"))
        self.uiSymbolLabel.setText(_translate("iosRouterConfigPageWidget", "Symbol:"))
        self.uiSymbolToolButton.setText(_translate("iosRouterConfigPageWidget", "&Browse..."))
        self.uiCategoryLabel.setText(_translate("iosRouterConfigPageWidget", "Category:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiGeneralPageWidget), _translate("iosRouterConfigPageWidget", "General"))
        self.uiRamLabel.setText(_translate("iosRouterConfigPageWidget", "RAM size:"))
        self.uiRamSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiNvramLabel.setText(_translate("iosRouterConfigPageWidget", "NVRAM size:"))
        self.uiNvramSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " KiB"))
        self.uiIomemLabel.setText(_translate("iosRouterConfigPageWidget", "I/O memory :"))
        self.uiIomemSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " %"))
        self.uiDisk0Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk0:"))
        self.uiDisk0SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiDisk1Label.setText(_translate("iosRouterConfigPageWidget", "PCMCIA disk1:"))
        self.uiDisk1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiAutoDeleteCheckBox.setText(_translate("iosRouterConfigPageWidget", "Automatically delete NVRAM and disk files"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiMemoriesPageWidget), _translate("iosRouterConfigPageWidget", "Memories and disks"))
        self.uiAdaptersGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Adapters"))
        self.uiSlot0Label.setText(_translate("iosRouterConfigPageWidget", "slot 0:"))
        self.uiSlot1Label.setText(_translate("iosRouterConfigPageWidget", "slot 1:"))
        self.uiSlot2Label.setText(_translate("iosRouterConfigPageWidget", "slot 2:"))
        self.uiSlot3Label.setText(_translate("iosRouterConfigPageWidget", "slot 3:"))
        self.uiSlot4Label.setText(_translate("iosRouterConfigPageWidget", "slot 4:"))
        self.uiSlot5Label.setText(_translate("iosRouterConfigPageWidget", "slot 5:"))
        self.uiSlot6Label.setText(_translate("iosRouterConfigPageWidget", "slot 6:"))
        self.uiWicsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "WICs"))
        self.uiWic0Label.setText(_translate("iosRouterConfigPageWidget", "wic 0:"))
        self.uiWic1Label.setText(_translate("iosRouterConfigPageWidget", "wic 1:"))
        self.uiWic2Label.setText(_translate("iosRouterConfigPageWidget", "wic 2:"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiSlotsPageWidget), _translate("iosRouterConfigPageWidget", "Slots"))
        self.uiSystemGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "System"))
        self.label.setText(_translate("iosRouterConfigPageWidget", "System ID:"))
        self.uiBaseMacLabel.setText(_translate("iosRouterConfigPageWidget", "Base MAC:"))
        self.uiOptimizationsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Optimisations"))
        self.uiExecAreaLabel.setText(_translate("iosRouterConfigPageWidget", "Exec area:"))
        self.uiExecAreaSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " MiB"))
        self.uiSparseMemoryCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable sparse memory support"))
        self.uiIdlepcLabel.setText(_translate("iosRouterConfigPageWidget", "Idle-PC:"))
        self.uiIdlemaxLabel.setText(_translate("iosRouterConfigPageWidget", "Idlemax:"))
        self.uiIdlesleepLabel.setText(_translate("iosRouterConfigPageWidget", "Idlesleep:"))
        self.uiMmapCheckBox.setText(_translate("iosRouterConfigPageWidget", "Enable mmap support"))
        self.uiIdlesleepSpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " ms"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiAdvancedPageWidget), _translate("iosRouterConfigPageWidget", "Advanced"))
        self.uiPowerSuppliesGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Power supplies"))
        self.uiPowerSupply2Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 2:"))
        self.uiPowerSupply1ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on"))
        self.uiPowerSupply1ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off"))
        self.uiPowerSupply1Label.setText(_translate("iosRouterConfigPageWidget", "Power supply 1:"))
        self.uiPowerSupply2ComboBox.setItemText(0, _translate("iosRouterConfigPageWidget", "on"))
        self.uiPowerSupply2ComboBox.setItemText(1, _translate("iosRouterConfigPageWidget", "off"))
        self.uiSensorsGroupBox.setTitle(_translate("iosRouterConfigPageWidget", "Temperature sensors"))
        self.uiSensor1Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller inlet:"))
        self.uiSensor1SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor2Label.setText(_translate("iosRouterConfigPageWidget", "I/0 controller outlet:"))
        self.uiSensor2SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor3Label.setText(_translate("iosRouterConfigPageWidget", "NPE inlet:"))
        self.uiSensor3SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiSensor4Label.setText(_translate("iosRouterConfigPageWidget", "NPE outlet:"))
        self.uiSensor4SpinBox.setSuffix(_translate("iosRouterConfigPageWidget", " C"))
        self.uiTabWidget.setTabText(self.uiTabWidget.indexOf(self.uiEnvironmentPageWidget), _translate("iosRouterConfigPageWidget", "Environment"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gns3.progress import Progress


def test_context():
    progress = Progress(min_duration=500)
    assert progress._minimum_duration == 500
    assert progress._enable

    with progress.context(min_duration=0):
        assert progress._minimum_duration == 0
        assert progress._enable
    assert progress._minimum_duration == 500

    with progress.context(enable=False):
        assert progress._minimum_duration == 500
        assert progress._enable is False
    assert progress._enable

    with progress.context(allow_cancel_query=True, cancel_button_text="Hello"):
        assert progress._cancel_button_text == "Hello"
        assert progress._allow_cancel_query is True
    assert progress._cancel_button_text == ""
    assert progress._allow_cancel_query is False



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/ui/doctor_dialog.ui'
#
# Created: Tue Jan 26 16:05:25 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_DoctorDialog(object):
    def setupUi(self, DoctorDialog):
        DoctorDialog.setObjectName("DoctorDialog")
        DoctorDialog.setWindowModality(QtCore.Qt.WindowModal)
        DoctorDialog.resize(607, 390)
        DoctorDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(DoctorDialog)
        self.verticalLayout.setContentsMargins(-1, -1, 12, -1)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(DoctorDialog)
        self.label.setTextFormat(QtCore.Qt.RichText)
        self.label.setScaledContents(False)
        self.label.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignTop)
        self.label.setWordWrap(True)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.uiDoctorResultTextEdit = QtWidgets.QTextEdit(DoctorDialog)
        self.uiDoctorResultTextEdit.setObjectName("uiDoctorResultTextEdit")
        self.verticalLayout.addWidget(self.uiDoctorResultTextEdit)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setSpacing(20)
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.uiOkButton = QtWidgets.QDialogButtonBox(DoctorDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiOkButton.sizePolicy().hasHeightForWidth())
        self.uiOkButton.setSizePolicy(sizePolicy)
        self.uiOkButton.setOrientation(QtCore.Qt.Horizontal)
        self.uiOkButton.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.uiOkButton.setObjectName("uiOkButton")
        self.horizontalLayout.addWidget(self.uiOkButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(DoctorDialog)
        QtCore.QMetaObject.connectSlotsByName(DoctorDialog)

    def retranslateUi(self, DoctorDialog):
        _translate = QtCore.QCoreApplication.translate
        DoctorDialog.setWindowTitle(_translate("DoctorDialog", "GNS3 Doctor"))
        self.label.setText(_translate("DoctorDialog", "<html><head/><body><p>This will list potential problem in your GNS3 installation:</p></body></html>"))
        self.uiDoctorResultTextEdit.setHtml(_translate("DoctorDialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Ubuntu\'; font-size:11pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-family:\'.SF NS Text\'; font-size:13pt; font-weight:600; font-style:italic;\">Starting checks...</span></p></body></html>"))

from . import resources_rc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/docker/ui/docker_vm_wizard.ui'
#
# Created by: PyQt5 UI code generator 5.5
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DockerVMWizard(object):

    def setupUi(self, DockerVMWizard):
        DockerVMWizard.setObjectName("DockerVMWizard")
        DockerVMWizard.resize(585, 424)
        DockerVMWizard.setModal(True)
        self.uiServerWizardPage = QtWidgets.QWizardPage()
        self.uiServerWizardPage.setObjectName("uiServerWizardPage")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.uiServerWizardPage)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiServerTypeGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiServerTypeGroupBox.setObjectName("uiServerTypeGroupBox")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.uiServerTypeGroupBox)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiRemoteRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiRemoteRadioButton.setChecked(True)
        self.uiRemoteRadioButton.setObjectName("uiRemoteRadioButton")
        self.horizontalLayout.addWidget(self.uiRemoteRadioButton)
        self.uiVMRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiVMRadioButton.setObjectName("uiVMRadioButton")
        self.horizontalLayout.addWidget(self.uiVMRadioButton)
        self.uiLocalRadioButton = QtWidgets.QRadioButton(self.uiServerTypeGroupBox)
        self.uiLocalRadioButton.setObjectName("uiLocalRadioButton")
        self.horizontalLayout.addWidget(self.uiLocalRadioButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout.addWidget(self.uiServerTypeGroupBox)
        self.uiRemoteServersGroupBox = QtWidgets.QGroupBox(self.uiServerWizardPage)
        self.uiRemoteServersGroupBox.setObjectName("uiRemoteServersGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiRemoteServersGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiLoadBalanceCheckBox = QtWidgets.QCheckBox(self.uiRemoteServersGroupBox)
        self.uiLoadBalanceCheckBox.setChecked(True)
        self.uiLoadBalanceCheckBox.setObjectName("uiLoadBalanceCheckBox")
        self.gridLayout_7.addWidget(self.uiLoadBalanceCheckBox, 0, 0, 1, 2)
        self.uiRemoteServersLabel = QtWidgets.QLabel(self.uiRemoteServersGroupBox)
        self.uiRemoteServersLabel.setObjectName("uiRemoteServersLabel")
        self.gridLayout_7.addWidget(self.uiRemoteServersLabel, 1, 0, 1, 1)
        self.uiRemoteServersComboBox = QtWidgets.QComboBox(self.uiRemoteServersGroupBox)
        self.uiRemoteServersComboBox.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteServersComboBox.sizePolicy().hasHeightForWidth())
        self.uiRemoteServersComboBox.setSizePolicy(sizePolicy)
        self.uiRemoteServersComboBox.setObjectName("uiRemoteServersComboBox")
        self.gridLayout_7.addWidget(self.uiRemoteServersComboBox, 1, 1, 1, 1)
        self.verticalLayout.addWidget(self.uiRemoteServersGroupBox)
        DockerVMWizard.addPage(self.uiServerWizardPage)
        self.uiImageWizardPage = QtWidgets.QWizardPage()
        self.uiImageWizardPage.setObjectName("uiImageWizardPage")
        self.gridLayout = QtWidgets.QGridLayout(self.uiImageWizardPage)
        self.gridLayout.setObjectName("gridLayout")
        self.uiImageListLabel = QtWidgets.QLabel(self.uiImageWizardPage)
        self.uiImageListLabel.setObjectName("uiImageListLabel")
        self.gridLayout.addWidget(self.uiImageListLabel, 0, 0, 1, 1)
        self.uiImageListComboBox = QtWidgets.QComboBox(self.uiImageWizardPage)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiImageListComboBox.sizePolicy().hasHeightForWidth())
        self.uiImageListComboBox.setSizePolicy(sizePolicy)
        self.uiImageListComboBox.setObjectName("uiImageListComboBox")
        self.gridLayout.addWidget(self.uiImageListComboBox, 0, 1, 1, 1)
        DockerVMWizard.addPage(self.uiImageWizardPage)

        self.retranslateUi(DockerVMWizard)
        QtCore.QMetaObject.connectSlotsByName(DockerVMWizard)

    def retranslateUi(self, DockerVMWizard):
        _translate = QtCore.QCoreApplication.translate
        DockerVMWizard.setWindowTitle(_translate("DockerVMWizard", "New Docker VM template"))
        self.uiServerWizardPage.setTitle(_translate("DockerVMWizard", "Server"))
        self.uiServerWizardPage.setSubTitle(_translate("DockerVMWizard", "Please choose a server type to run your new Docker VM."))
        self.uiServerTypeGroupBox.setTitle(_translate("DockerVMWizard", "Server type"))
        self.uiRemoteRadioButton.setText(_translate("DockerVMWizard", "Remote"))
        self.uiVMRadioButton.setText(_translate("DockerVMWizard", "GNS3 VM"))
        self.uiLocalRadioButton.setText(_translate("DockerVMWizard", "Local"))
        self.uiRemoteServersGroupBox.setTitle(_translate("DockerVMWizard", "Remote servers"))
        self.uiLoadBalanceCheckBox.setText(_translate("DockerVMWizard", "Load balance across all available remote servers"))
        self.uiRemoteServersLabel.setText(_translate("DockerVMWizard", "Run on server:"))
        self.uiImageWizardPage.setTitle(_translate("DockerVMWizard", "Docker Virtual Machine"))
        self.uiImageWizardPage.setSubTitle(_translate("DockerVMWizard", "Please choose a Docker virtual machine from the list."))
        self.uiImageListLabel.setText(_translate("DockerVMWizard", "Image list:"))
